# vim: set fileencoding=utf-8:
#
# GPIO Zero: a library for controlling the Raspberry Pi's GPIO pins
#
# Copyright (c) 2016-2021 Dave Jones <dave@waveform.org.uk>
# Copyright (c) 2018-2019 Ben Nuttall <ben@bennuttall.com>
# Copyright (c) 2016 Andrew Scheller <github@loowis.durge.org>
#
# SPDX-License-Identifier: BSD-3-Clause

from __future__ import (
    unicode_literals,
    absolute_import,
    print_function,
    division,
    )
str = type('')


import sys
from time import sleep, time
try:
    from math import isclose
except ImportError:
    from gpiozero.compat import isclose

import pytest
from colorzero import Color, Red, Green, Blue

from gpiozero import *
from gpiozero.tones import Tone


def test_output_initial_values(mock_factory, pwm):
    pin = mock_factory.pin(2)
    with OutputDevice(2, initial_value=False) as device:
        assert repr(device).startswith('<gpiozero.OutputDevice object')
        assert pin.function == 'output'
        assert not pin.state
    assert repr(device) == '<gpiozero.OutputDevice object closed>'
    with OutputDevice(2, initial_value=True) as device:
        assert pin.state
        state = pin.state
    with OutputDevice(2, initial_value=None) as device:
        assert state == pin.state

def test_output_write_active_high(mock_factory):
    pin = mock_factory.pin(2)
    with OutputDevice(2) as device:
        device.on()
        assert pin.state
        device.off()
        assert not pin.state

def test_output_write_active_low(mock_factory):
    pin = mock_factory.pin(2)
    with OutputDevice(2, active_high=False) as device:
        device.on()
        assert not pin.state
        device.off()
        assert pin.state

def test_output_write_closed(mock_factory):
    with OutputDevice(2) as device:
        device.close()
        assert device.closed
        device.close()
        assert device.closed
        with pytest.raises(GPIODeviceClosed):
            device.on()

def test_output_write_silly(mock_factory):
    pin = mock_factory.pin(2)
    with OutputDevice(2) as device:
        pin.function = 'input'
        with pytest.raises(AttributeError):
            device.on()

def test_output_value(mock_factory):
    pin = mock_factory.pin(2)
    with OutputDevice(2) as device:
        assert not device.value
        assert not pin.state
        device.on()
        assert device.value
        assert pin.state
        device.value = False
        assert not device.value
        assert not pin.state

def test_output_digital_toggle(mock_factory):
    pin = mock_factory.pin(2)
    with DigitalOutputDevice(2) as device:
        assert repr(device).startswith('<gpiozero.DigitalOutputDevice object')
        assert not device.value
        assert not pin.state
        device.toggle()
        assert device.value
        assert pin.state
        device.toggle()
        assert not device.value
        assert not pin.state

@pytest.mark.skipif(hasattr(sys, 'pypy_version_info'),
                    reason='timing is too random on pypy')
def test_output_blink_background(mock_factory):
    pin = mock_factory.pin(4)
    with DigitalOutputDevice(4) as device:
        start = time()
        device.blink(0.1, 0.1, n=2)
        assert isclose(time() - start, 0, abs_tol=0.05)
        device._blink_thread.join() # naughty, but ensures no arbitrary waits in the test
        assert isclose(time() - start, 0.4, abs_tol=0.05)
        pin.assert_states_and_times([
            (0.0, False),
            (0.0, True),
            (0.1, False),
            (0.1, True),
            (0.1, False)
            ])

@pytest.mark.skipif(hasattr(sys, 'pypy_version_info'),
                    reason='timing is too random on pypy')
def test_output_blink_foreground(mock_factory):
    pin = mock_factory.pin(4)
    with DigitalOutputDevice(4) as device:
        start = time()
        device.blink(0.1, 0.1, n=2, background=False)
        assert isclose(time() - start, 0.4, abs_tol=0.05)
        pin.assert_states_and_times([
            (0.0, False),
            (0.0, True),
            (0.1, False),
            (0.1, True),
            (0.1, False)
            ])

def test_output_blink_interrupt_on(mock_factory):
    pin = mock_factory.pin(4)
    with DigitalOutputDevice(4) as device:
        device.blink(1, 0.1)
        sleep(0.2)
        device.off() # should interrupt while on
        pin.assert_states([False, True, False])

def test_output_blink_interrupt_off(mock_factory):
    pin = mock_factory.pin(4)
    with DigitalOutputDevice(4) as device:
        device.blink(0.1, 1)
        sleep(0.2)
        device.off() # should interrupt while off
        pin.assert_states([False, True, False])

def test_output_pwm_bad_initial_value(mock_factory):
    with pytest.raises(ValueError):
        PWMOutputDevice(2, initial_value=2)

def test_output_pwm_not_supported(mock_factory):
    with pytest.raises(AttributeError):
        PWMOutputDevice(2)

def test_output_pwm_states(mock_factory, pwm):
    pin = mock_factory.pin(4)
    with PWMOutputDevice(4) as device:
        assert repr(device).startswith('<gpiozero.PWMOutputDevice object')
        device.value = 0.1
        device.value = 0.2
        device.value = 0.0
        pin.assert_states([0.0, 0.1, 0.2, 0.0])

def test_output_pwm_read(mock_factory, pwm):
    pin = mock_factory.pin(2)
    with PWMOutputDevice(2, frequency=100) as device:
        assert device.frequency == 100
        device.value = 0.1
        assert isclose(device.value, 0.1)
        assert isclose(pin.state, 0.1)
        assert device.is_active
        device.frequency = None
        assert not device.value
        assert not device.is_active
        assert device.frequency is None

def test_output_pwm_write(mock_factory, pwm):
    pin = mock_factory.pin(4)
    with PWMOutputDevice(4) as device:
        device.on()
        device.off()
        pin.assert_states([False, True, False])

def test_output_pwm_toggle(mock_factory, pwm):
    pin = mock_factory.pin(4)
    with PWMOutputDevice(4) as device:
        device.toggle()
        device.value = 0.5
        device.value = 0.1
        device.toggle()
        device.off()
        pin.assert_states([False, True, 0.5, 0.1, 0.9, False])

def test_output_pwm_active_high_read(mock_factory, pwm):
    pin = mock_factory.pin(2)
    with PWMOutputDevice(2, active_high=False) as device:
        device.value = 0.1
        assert isclose(device.value, 0.1)
        assert isclose(pin.state, 0.9)
        device.frequency = None
        assert device.value

def test_output_pwm_bad_value(mock_factory, pwm):
    pin = mock_factory.pin(2)
    with PWMOutputDevice(2) as device:
        with pytest.raises(ValueError):
            device.value = 2

def test_output_pwm_write_closed(mock_factory, pwm):
    pin = mock_factory.pin(2)
    with PWMOutputDevice(2) as device:
        device.close()
        with pytest.raises(GPIODeviceClosed):
            device.on()

def test_output_pwm_write_silly(mock_factory, pwm):
    pin = mock_factory.pin(2)
    with PWMOutputDevice(2) as device:
        pin.function = 'input'
        with pytest.raises(AttributeError):
            device.off()

@pytest.mark.skipif(hasattr(sys, 'pypy_version_info'),
                    reason='timing is too random on pypy')
def test_output_pwm_blink_background(mock_factory, pwm):
    pin = mock_factory.pin(4)
    with PWMOutputDevice(4) as device:
        start = time()
        device.blink(0.1, 0.1, n=2)
        assert isclose(time() - start, 0, abs_tol=0.05)
        device._blink_thread.join()
        assert isclose(time() - start, 0.4, abs_tol=0.05)
        pin.assert_states_and_times([
            (0.0, 0),
            (0.0, 1),
            (0.1, 0),
            (0.1, 1),
            (0.1, 0)
            ])

@pytest.mark.skipif(hasattr(sys, 'pypy_version_info'),
                    reason='timing is too random on pypy')
def test_output_pwm_blink_foreground(mock_factory, pwm):
    pin = mock_factory.pin(4)
    with PWMOutputDevice(4) as device:
        start = time()
        device.blink(0.1, 0.1, n=2, background=False)
        assert isclose(time() - start, 0.4, abs_tol=0.05)
        pin.assert_states_and_times([
            (0.0, 0),
            (0.0, 1),
            (0.1, 0),
            (0.1, 1),
            (0.1, 0)
            ])

@pytest.mark.skipif(hasattr(sys, 'pypy_version_info'),
                    reason='timing is too random on pypy')
def test_output_pwm_fade_background(mock_factory, pwm):
    pin = mock_factory.pin(4)
    with PWMOutputDevice(4) as device:
        start = time()
        device.blink(0, 0, 0.2, 0.2, n=2)
        assert isclose(time() - start, 0, abs_tol=0.05)
        device._blink_thread.join()
        assert isclose(time() - start, 0.8, abs_tol=0.05)
        pin.assert_states_and_times([
            (0.0, 0),
            (0.04, 0.2),
            (0.04, 0.4),
            (0.04, 0.6),
            (0.04, 0.8),
            (0.04, 1),
            (0.04, 0.8),
            (0.04, 0.6),
            (0.04, 0.4),
            (0.04, 0.2),
            (0.04, 0),
            (0.04, 0.2),
            (0.04, 0.4),
            (0.04, 0.6),
            (0.04, 0.8),
            (0.04, 1),
            (0.04, 0.8),
            (0.04, 0.6),
            (0.04, 0.4),
            (0.04, 0.2),
            (0.04, 0),
            ])

@pytest.mark.skipif(hasattr(sys, 'pypy_version_info'),
                    reason='timing is too random on pypy')
def test_output_pwm_fade_foreground(mock_factory, pwm):
    pin = mock_factory.pin(4)
    with PWMOutputDevice(4) as device:
        start = time()
        device.blink(0, 0, 0.2, 0.2, n=2, background=False)
        assert isclose(time() - start, 0.8, abs_tol=0.05)
        pin.assert_states_and_times([
            (0.0, 0),
            (0.04, 0.2),
            (0.04, 0.4),
            (0.04, 0.6),
            (0.04, 0.8),
            (0.04, 1),
            (0.04, 0.8),
            (0.04, 0.6),
            (0.04, 0.4),
            (0.04, 0.2),
            (0.04, 0),
            (0.04, 0.2),
            (0.04, 0.4),
            (0.04, 0.6),
            (0.04, 0.8),
            (0.04, 1),
            (0.04, 0.8),
            (0.04, 0.6),
            (0.04, 0.4),
            (0.04, 0.2),
            (0.04, 0),
            ])

@pytest.mark.skipif(hasattr(sys, 'pypy_version_info'),
                    reason='timing is too random on pypy')
def test_output_pwm_pulse_background(mock_factory, pwm):
    pin = mock_factory.pin(4)
    with PWMOutputDevice(4) as device:
        start = time()
        device.pulse(0.2, 0.2, n=2)
        assert isclose(time() - start, 0, abs_tol=0.05)
        device._blink_thread.join()
        assert isclose(time() - start, 0.8, abs_tol=0.05)
        pin.assert_states_and_times([
            (0.0, 0),
            (0.04, 0.2),
            (0.04, 0.4),
            (0.04, 0.6),
            (0.04, 0.8),
            (0.04, 1),
            (0.04, 0.8),
            (0.04, 0.6),
            (0.04, 0.4),
            (0.04, 0.2),
            (0.04, 0),
            (0.04, 0.2),
            (0.04, 0.4),
            (0.04, 0.6),
            (0.04, 0.8),
            (0.04, 1),
            (0.04, 0.8),
            (0.04, 0.6),
            (0.04, 0.4),
            (0.04, 0.2),
            (0.04, 0),
            ])

@pytest.mark.skipif(hasattr(sys, 'pypy_version_info'),
                    reason='timing is too random on pypy')
def test_output_pwm_pulse_foreground(mock_factory, pwm):
    pin = mock_factory.pin(4)
    with PWMOutputDevice(4) as device:
        start = time()
        device.pulse(0.2, 0.2, n=2, background=False)
        assert isclose(time() - start, 0.8, abs_tol=0.05)
        pin.assert_states_and_times([
            (0.0, 0),
            (0.04, 0.2),
            (0.04, 0.4),
            (0.04, 0.6),
            (0.04, 0.8),
            (0.04, 1),
            (0.04, 0.8),
            (0.04, 0.6),
            (0.04, 0.4),
            (0.04, 0.2),
            (0.04, 0),
            (0.04, 0.2),
            (0.04, 0.4),
            (0.04, 0.6),
            (0.04, 0.8),
            (0.04, 1),
            (0.04, 0.8),
            (0.04, 0.6),
            (0.04, 0.4),
            (0.04, 0.2),
            (0.04, 0),
            ])

def test_output_pwm_blink_interrupt(mock_factory, pwm):
    pin = mock_factory.pin(4)
    with PWMOutputDevice(4) as device:
        device.blink(1, 0.1)
        sleep(0.2)
        device.off() # should interrupt while on
        pin.assert_states([0, 1, 0])

def test_rgbled_missing_pins(mock_factory):
    with pytest.raises(GPIOPinMissing):
        RGBLED()

def test_rgbled_initial_value(mock_factory, pwm):
    r, g, b = (mock_factory.pin(i) for i in (1, 2, 3))
    with RGBLED(1, 2, 3, initial_value=(0.1, 0.2, 0)) as led:
        assert repr(led).startswith('<gpiozero.RGBLED object')
        assert r.frequency
        assert g.frequency
        assert b.frequency
        assert isclose(r.state, 0.1)
        assert isclose(g.state, 0.2)
        assert isclose(b.state, 0.0)

def test_rgbled_initial_value_nonpwm(mock_factory):
    r, g, b = (mock_factory.pin(i) for i in (1, 2, 3))
    with RGBLED(1, 2, 3, pwm=False, initial_value=(0, 1, 1)) as led:
        assert r.state == 0
        assert g.state == 1
        assert b.state == 1

def test_rgbled_initial_bad_value(mock_factory, pwm):
    with pytest.raises(ValueError):
        RGBLED(1, 2, 3, initial_value=(0.1, 0.2, 1.2))

def test_rgbled_initial_bad_value_nonpwm(mock_factory):
    with pytest.raises(ValueError):
        RGBLED(1, 2, 3, pwm=False, initial_value=(0.1, 0.2, 0))

def test_rgbled_value(mock_factory, pwm):
    with RGBLED(1, 2, 3) as led:
        assert isinstance(led._leds[0], PWMLED)
        assert isinstance(led._leds[1], PWMLED)
        assert isinstance(led._leds[2], PWMLED)
        assert not led.is_active
        assert led.value == (0, 0, 0)
        led.on()
        assert led.is_active
        assert led.value == (1, 1, 1)
        led.off()
        assert not led.is_active
        assert led.value == (0, 0, 0)
        led.value = (0.5, 0.5, 0.5)
        assert led.is_active
        assert led.value == (0.5, 0.5, 0.5)

def test_rgbled_value_nonpwm(mock_factory):
    with RGBLED(1, 2, 3, pwm=False) as led:
        assert isinstance(led._leds[0], LED)
        assert isinstance(led._leds[1], LED)
        assert isinstance(led._leds[2], LED)
        assert not led.is_active
        assert led.value == (0, 0, 0)
        led.on()
        assert led.is_active
        assert led.value == (1, 1, 1)
        led.off()
        assert not led.is_active
        assert led.value == (0, 0, 0)

def test_rgbled_bad_value(mock_factory, pwm):
    with RGBLED(1, 2, 3) as led:
        with pytest.raises(ValueError):
            led.value = (2, 0, 0)
    with RGBLED(1, 2, 3) as led:
        with pytest.raises(ValueError):
            led.value = (0, -1, 0)

def test_rgbled_bad_value_nonpwm(mock_factory):
    with RGBLED(1, 2, 3, pwm=False) as led:
        with pytest.raises(ValueError):
            led.value = (2, 0, 0)
    with RGBLED(1, 2, 3, pwm=False) as led:
        with pytest.raises(ValueError):
            led.value = (0, -1, 0)
    with RGBLED(1, 2, 3, pwm=False) as led:
        with pytest.raises(ValueError):
            led.value = (0.5, 0, 0)
    with RGBLED(1, 2, 3, pwm=False) as led:
        with pytest.raises(ValueError):
            led.value = (0, 0.5, 0)
    with RGBLED(1, 2, 3, pwm=False) as led:
        with pytest.raises(ValueError):
            led.value = (0, 0, 0.5)

def test_rgbled_toggle(mock_factory, pwm):
    with RGBLED(1, 2, 3) as led:
        assert not led.is_active
        assert led.value == (0, 0, 0)
        led.toggle()
        assert led.is_active
        assert led.value == (1, 1, 1)
        led.toggle()
        assert not led.is_active
        assert led.value == (0, 0, 0)

def test_rgbled_toggle_nonpwm(mock_factory):
    with RGBLED(1, 2, 3, pwm=False) as led:
        assert not led.is_active
        assert led.value == (0, 0, 0)
        led.toggle()
        assert led.is_active
        assert led.value == (1, 1, 1)
        led.toggle()
        assert not led.is_active
        assert led.value == (0, 0, 0)

def test_rgbled_bad_color_value_nonpwm(mock_factory):
    with RGBLED(1, 2, 3, pwm=False) as led:
        with pytest.raises(ValueError):
            led.color = (0.5, 0, 0)
        with pytest.raises(ValueError):
            led.color = (0, 1.5, 0)
        with pytest.raises(ValueError):
            led.color = (0, 0, -1)

def test_rgbled_bad_color_value_pwm(mock_factory, pwm):
    with RGBLED(1, 2, 3) as led:
        with pytest.raises(ValueError):
            led.color = (0, 1.5, 0)
        with pytest.raises(ValueError):
            led.color = (0, 0, -1)
        with pytest.raises(ValueError):
            led.green = 1.5
        with pytest.raises(ValueError):
            led.blue = -1

def test_rgbled_color_value_nonpwm(mock_factory):
    with RGBLED(1, 2, 3, pwm=False) as led:
        assert led.value == (0, 0, 0)
        assert led.red == 0
        assert led.green == 0
        assert led.blue == 0
        led.on()
        assert led.value == (1, 1, 1)
        assert led.color == (1, 1, 1)
        assert led.red == 1
        assert led.green == 1
        assert led.blue == 1
        led.color = (0, 1, 0)
        assert led.value == (0, 1, 0)
        assert led.red == 0
        led.red = 1
        assert led.value == (1, 1, 0)
        assert led.red == 1
        assert led.green == 1
        assert led.blue == 0
        led.green = 0
        led.blue = 1
        assert led.value == (1, 0, 1)

def test_rgbled_color_value_pwm(mock_factory, pwm):
    with RGBLED(1, 2, 3) as led:
        assert led.value == (0, 0, 0)
        assert led.red == 0
        assert led.green == 0
        assert led.blue == 0
        led.on()
        assert led.value == (1, 1, 1)
        assert led.color == (1, 1, 1)
        assert led.red == 1
        assert led.green == 1
        assert led.blue == 1
        led.color = (0.2, 0.5, 0.8)
        assert led.value == (0.2, 0.5, 0.8)
        assert led.red == 0.2
        led.red = 0.5
        assert led.value == (0.5, 0.5, 0.8)
        assert led.red == 0.5
        assert led.green == 0.5
        assert led.blue == 0.8
        led.green = 0.9
        led.blue = 0.4
        assert led.value == (0.5, 0.9, 0.4)

def test_rgbled_bad_rgb_property_nonpwm(mock_factory):
    with RGBLED(1, 2, 3, pwm=False) as led:
        with pytest.raises(ValueError):
            led.red = 0.1
        with pytest.raises(ValueError):
            led.green = 0.5
        with pytest.raises(ValueError):
            led.blue = 0.9
        with pytest.raises(ValueError):
            led.red = Red(0.1)
        with pytest.raises(ValueError):
            led.green = Green(0.5)
        with pytest.raises(ValueError):
            led.blue = Blue(0.9)

def test_rgbled_bad_rgb_property_pwm(mock_factory, pwm):
    with RGBLED(1, 2, 3) as led:
        with pytest.raises(ValueError):
            led.red = 1.5
        with pytest.raises(ValueError):
            led.green = 2
        with pytest.raises(ValueError):
            led.blue = -1
        with pytest.raises(ValueError):
            led.red = Red(1.5)
        with pytest.raises(ValueError):
            led.green = Green(2)
        with pytest.raises(ValueError):
            led.blue = Blue(-1)

def test_rgbled_rgb_property_nonpwm(mock_factory):
    with RGBLED(1, 2, 3, pwm=False) as led:
        assert led.value == (0, 0, 0)
        led.red = Red(0)
        assert led.value == (0, 0, 0)
        led.red = Red(1)
        assert led.value == (1, 0, 0)
        led.green = Green(1)
        assert led.value == (1, 1, 0)
        led.blue = Blue(1)
        assert led.value == (1, 1, 1)

def test_rgbled_rgb_property_pwm(mock_factory, pwm):
    with RGBLED(1, 2, 3) as led:
        assert led.value == (0, 0, 0)
        led.red = Red(0)
        assert led.value == (0, 0, 0)
        led.red = Red(0.5)
        assert led.value == (0.5, 0, 0)
        led.green = Green(0.5)
        assert led.value == (0.5, 0.5, 0)
        led.blue = Blue(0.5)
        assert led.value == (0.5, 0.5, 0.5)

def test_rgbled_bad_color_name_nonpwm(mock_factory):
    with RGBLED(1, 2, 3, pwm=False) as led:
        with pytest.raises(ValueError):
            led.color = Color('green')  # html 'green' is (0, ~0.5, 0)
        with pytest.raises(ValueError):
            led.color = Color(0.5, 0, 0)
        with pytest.raises(ValueError):
            led.color = Color(250, 0, 0)

def test_rgbled_color_name_nonpwm(mock_factory):
    with RGBLED(1, 2, 3, pwm=False) as led:
        assert led.value == (0, 0, 0)
        led.color = Color('white')
        assert led.value == (1, 1, 1)
        led.color = Color('black')
        assert led.value == (0, 0, 0)
        led.color = Color('red')
        assert led.value == (1, 0, 0)
        led.color = Color('lime')  # html 'green' is (0, 0.5, 0)
        assert led.value == (0, 1, 0)
        led.color = Color('blue')
        assert led.value == (0, 0, 1)
        led.color = Color('cyan')
        assert led.value == (0, 1, 1)
        led.color = Color('magenta')
        assert led.value == (1, 0, 1)
        led.color = Color('yellow')
        assert led.value == (1, 1, 0)

def test_rgbled_color_name_pwm(mock_factory, pwm):
    with RGBLED(1, 2, 3) as led:
        assert led.value == (0, 0, 0)
        led.color = Color('white')
        assert led.value == (1, 1, 1)
        led.color = Color('green')
        assert led.value == (0, 0.5019607843137255, 0)
        led.color = Color('chocolate')
        assert led.value == (0.8235294117647058, 0.4117647058823529, 0.11764705882352941)
        led.color = Color('purple')
        assert led.value == (0.5019607843137255, 0.0, 0.5019607843137255)

def test_rgbled_blink_nonpwm(mock_factory):
    with RGBLED(1, 2, 3, pwm=False) as led:
        with pytest.raises(ValueError):
            led.blink(fade_in_time=1)
        with pytest.raises(ValueError):
            led.blink(fade_out_time=1)

@pytest.mark.skipif(hasattr(sys, 'pypy_version_info'),
                    reason='timing is too random on pypy')
def test_rgbled_blink_background(mock_factory, pwm):
    r, g, b = (mock_factory.pin(i) for i in (4, 5, 6))
    with RGBLED(1, 2, 3) as led:
        start = time()
        led.blink(0.1, 0.1, n=2)
        assert isclose(time() - start, 0, abs_tol=0.05)
        led._blink_thread.join()
        assert isclose(time() - start, 0.4, abs_tol=0.05)
        expected = [
            (0.0, 0),
            (0.0, 1),
            (0.1, 0),
            (0.1, 1),
            (0.1, 0)
            ]
        r.assert_states_and_times(expected)
        g.assert_states_and_times(expected)
        b.assert_states_and_times(expected)

@pytest.mark.skipif(hasattr(sys, 'pypy_version_info'),
                    reason='timing is too random on pypy')
def test_rgbled_blink_background_nonpwm(mock_factory):
    r, g, b = (mock_factory.pin(i) for i in (4, 5, 6))
    with RGBLED(1, 2, 3, pwm=False) as led:
        start = time()
        led.blink(0.1, 0.1, n=2)
        assert isclose(time() - start, 0, abs_tol=0.05)
        led._blink_thread.join()
        assert isclose(time() - start, 0.4, abs_tol=0.05)
        expected = [
            (0.0, 0),
            (0.0, 1),
            (0.1, 0),
            (0.1, 1),
            (0.1, 0)
            ]
        r.assert_states_and_times(expected)
        g.assert_states_and_times(expected)
        b.assert_states_and_times(expected)

@pytest.mark.skipif(hasattr(sys, 'pypy_version_info'),
                    reason='timing is too random on pypy')
def test_rgbled_blink_foreground(mock_factory, pwm):
    r, g, b = (mock_factory.pin(i) for i in (4, 5, 6))
    with RGBLED(1, 2, 3) as led:
        start = time()
        led.blink(0.1, 0.1, n=2, background=False)
        assert isclose(time() - start, 0.4, abs_tol=0.05)
        expected = [
            (0.0, 0),
            (0.0, 1),
            (0.1, 0),
            (0.1, 1),
            (0.1, 0)
            ]
        r.assert_states_and_times(expected)
        g.assert_states_and_times(expected)
        b.assert_states_and_times(expected)

@pytest.mark.skipif(hasattr(sys, 'pypy_version_info'),
                    reason='timing is too random on pypy')
def test_rgbled_blink_foreground_nonpwm(mock_factory):
    r, g, b = (mock_factory.pin(i) for i in (4, 5, 6))
    with RGBLED(1, 2, 3, pwm=False) as led:
        start = time()
        led.blink(0.1, 0.1, n=2, background=False)
        assert isclose(time() - start, 0.4, abs_tol=0.05)
        expected = [
            (0.0, 0),
            (0.0, 1),
            (0.1, 0),
            (0.1, 1),
            (0.1, 0)
            ]
        r.assert_states_and_times(expected)
        g.assert_states_and_times(expected)
        b.assert_states_and_times(expected)

@pytest.mark.skipif(hasattr(sys, 'pypy_version_info'),
                    reason='timing is too random on pypy')
def test_rgbled_fade_background(mock_factory, pwm):
    r, g, b = (mock_factory.pin(i) for i in (4, 5, 6))
    with RGBLED(1, 2, 3) as led:
        start = time()
        led.blink(0, 0, 0.2, 0.2, n=2)
        assert isclose(time() - start, 0, abs_tol=0.05)
        led._blink_thread.join()
        assert isclose(time() - start, 0.8, abs_tol=0.05)
        expected = [
            (0.0, 0),
            (0.04, 0.2),
            (0.04, 0.4),
            (0.04, 0.6),
            (0.04, 0.8),
            (0.04, 1),
            (0.04, 0.8),
            (0.04, 0.6),
            (0.04, 0.4),
            (0.04, 0.2),
            (0.04, 0),
            (0.04, 0.2),
            (0.04, 0.4),
            (0.04, 0.6),
            (0.04, 0.8),
            (0.04, 1),
            (0.04, 0.8),
            (0.04, 0.6),
            (0.04, 0.4),
            (0.04, 0.2),
            (0.04, 0),
            ]
        r.assert_states_and_times(expected)
        g.assert_states_and_times(expected)
        b.assert_states_and_times(expected)

def test_rgbled_fade_background_nonpwm(mock_factory):
    r, g, b = (mock_factory.pin(i) for i in (1, 2, 3))
    with RGBLED(1, 2, 3, pwm=False) as led:
        with pytest.raises(ValueError):
            led.blink(0, 0, 0.2, 0, n=2)
        with pytest.raises(ValueError):
            led.blink(0, 0, 0, 0.2, n=2)

@pytest.mark.skipif(hasattr(sys, 'pypy_version_info'),
                    reason='timing is too random on pypy')
def test_rgbled_fade_foreground(mock_factory, pwm):
    r, g, b = (mock_factory.pin(i) for i in (4, 5, 6))
    with RGBLED(1, 2, 3) as led:
        start = time()
        led.blink(0, 0, 0.2, 0.2, n=2, background=False)
        assert isclose(time() - start, 0.8, abs_tol=0.05)
        expected = [
            (0.0, 0),
            (0.04, 0.2),
            (0.04, 0.4),
            (0.04, 0.6),
            (0.04, 0.8),
            (0.04, 1),
            (0.04, 0.8),
            (0.04, 0.6),
            (0.04, 0.4),
            (0.04, 0.2),
            (0.04, 0),
            (0.04, 0.2),
            (0.04, 0.4),
            (0.04, 0.6),
            (0.04, 0.8),
            (0.04, 1),
            (0.04, 0.8),
            (0.04, 0.6),
            (0.04, 0.4),
            (0.04, 0.2),
            (0.04, 0),
            ]
        r.assert_states_and_times(expected)
        g.assert_states_and_times(expected)
        b.assert_states_and_times(expected)

def test_rgbled_fade_foreground_nonpwm(mock_factory):
    r, g, b = (mock_factory.pin(i) for i in (1, 2, 3))
    with RGBLED(1, 2, 3, pwm=False) as led:
        with pytest.raises(ValueError):
            led.blink(0, 0, 0.2, 0.2, n=2, background=False)

@pytest.mark.skipif(hasattr(sys, 'pypy_version_info'),
                    reason='timing is too random on pypy')
def test_rgbled_pulse_background(mock_factory, pwm):
    r, g, b = (mock_factory.pin(i) for i in (4, 5, 6))
    with RGBLED(1, 2, 3) as led:
        start = time()
        led.pulse(0.2, 0.2, n=2)
        assert isclose(time() - start, 0, abs_tol=0.05)
        led._blink_thread.join()
        assert isclose(time() - start, 0.8, abs_tol=0.05)
        expected = [
            (0.0, 0),
            (0.04, 0.2),
            (0.04, 0.4),
            (0.04, 0.6),
            (0.04, 0.8),
            (0.04, 1),
            (0.04, 0.8),
            (0.04, 0.6),
            (0.04, 0.4),
            (0.04, 0.2),
            (0.04, 0),
            (0.04, 0.2),
            (0.04, 0.4),
            (0.04, 0.6),
            (0.04, 0.8),
            (0.04, 1),
            (0.04, 0.8),
            (0.04, 0.6),
            (0.04, 0.4),
            (0.04, 0.2),
            (0.04, 0),
            ]
        r.assert_states_and_times(expected)
        g.assert_states_and_times(expected)
        b.assert_states_and_times(expected)

def test_rgbled_pulse_background_nonpwm(mock_factory):
    r, g, b = (mock_factory.pin(i) for i in (1, 2, 3))
    with RGBLED(1, 2, 3, pwm=False) as led:
        with pytest.raises(ValueError):
            led.pulse(0.2, 0.2, n=2)

@pytest.mark.skipif(hasattr(sys, 'pypy_version_info'),
                    reason='timing is too random on pypy')
def test_rgbled_pulse_foreground(mock_factory, pwm):
    r, g, b = (mock_factory.pin(i) for i in (4, 5, 6))
    with RGBLED(1, 2, 3) as led:
        start = time()
        led.pulse(0.2, 0.2, n=2, background=False)
        assert isclose(time() - start, 0.8, abs_tol=0.05)
        expected = [
            (0.0, 0),
            (0.04, 0.2),
            (0.04, 0.4),
            (0.04, 0.6),
            (0.04, 0.8),
            (0.04, 1),
            (0.04, 0.8),
            (0.04, 0.6),
            (0.04, 0.4),
            (0.04, 0.2),
            (0.04, 0),
            (0.04, 0.2),
            (0.04, 0.4),
            (0.04, 0.6),
            (0.04, 0.8),
            (0.04, 1),
            (0.04, 0.8),
            (0.04, 0.6),
            (0.04, 0.4),
            (0.04, 0.2),
            (0.04, 0),
            ]
        r.assert_states_and_times(expected)
        g.assert_states_and_times(expected)
        b.assert_states_and_times(expected)

def test_rgbled_pulse_foreground_nonpwm(mock_factory):
    r, g, b = (mock_factory.pin(i) for i in (1, 2, 3))
    with RGBLED(1, 2, 3, pwm=False) as led:
        with pytest.raises(ValueError):
            led.pulse(0.2, 0.2, n=2, background=False)

def test_rgbled_blink_interrupt(mock_factory, pwm):
    r, g, b = (mock_factory.pin(i) for i in (4, 5, 6))
    with RGBLED(1, 2, 3) as led:
        led.blink(1, 0.1)
        sleep(0.2)
        led.off() # should interrupt while on
        r.assert_states([0, 1, 0])
        g.assert_states([0, 1, 0])
        b.assert_states([0, 1, 0])

def test_rgbled_blink_interrupt_nonpwm(mock_factory):
    r, g, b = (mock_factory.pin(i) for i in (4, 5, 6))
    with RGBLED(1, 2, 3, pwm=False) as led:
        led.blink(1, 0.1)
        sleep(0.2)
        led.off() # should interrupt while on
        r.assert_states([0, 1, 0])
        g.assert_states([0, 1, 0])
        b.assert_states([0, 1, 0])

def test_rgbled_close(mock_factory, pwm):
    r, g, b = (mock_factory.pin(i) for i in (1, 2, 3))
    with RGBLED(1, 2, 3) as led:
        assert not led.closed
        led.close()
        assert led.closed
        led.close()
        assert led.closed

def test_rgbled_close_nonpwm(mock_factory):
    r, g, b = (mock_factory.pin(i) for i in (1, 2, 3))
    with RGBLED(1, 2, 3, pwm=False) as led:
        assert not led.closed
        led.close()
        assert led.closed
        led.close()
        assert led.closed

def test_motor_bad_init(mock_factory):
    with pytest.raises(GPIOPinMissing):
        Motor()
    with pytest.raises(GPIOPinMissing):
        Motor(2)
    with pytest.raises(GPIOPinMissing):
        Motor(forward=2)
    with pytest.raises(GPIOPinMissing):
        Motor(backward=2)
    with pytest.raises(TypeError):
        Motor(a=2, b=3)

def test_motor_pins(mock_factory, pwm):
    f = mock_factory.pin(1)
    b = mock_factory.pin(2)
    with Motor(1, 2) as motor:
        assert repr(motor).startswith('<gpiozero.Motor object')
        assert motor.forward_device.pin is f
        assert isinstance(motor.forward_device, PWMOutputDevice)
        assert motor.backward_device.pin is b
        assert isinstance(motor.backward_device, PWMOutputDevice)

def test_motor_pins_nonpwm(mock_factory):
    f = mock_factory.pin(1)
    b = mock_factory.pin(2)
    with Motor(1, 2, pwm=False) as motor:
        assert motor.forward_device.pin is f
        assert isinstance(motor.forward_device, DigitalOutputDevice)
        assert motor.backward_device.pin is b
        assert isinstance(motor.backward_device, DigitalOutputDevice)

def test_motor_close(mock_factory, pwm):
    f = mock_factory.pin(1)
    b = mock_factory.pin(2)
    with Motor(1, 2) as motor:
        motor.close()
        assert motor.closed
        assert motor.forward_device.pin is None
        assert motor.backward_device.pin is None
        motor.close()
        assert motor.closed

def test_motor_close_nonpwm(mock_factory):
    f = mock_factory.pin(1)
    b = mock_factory.pin(2)
    with Motor(1, 2, pwm=False) as motor:
        motor.close()
        assert motor.closed
        assert motor.forward_device.pin is None
        assert motor.backward_device.pin is None

def test_motor_value(mock_factory, pwm):
    f = mock_factory.pin(1)
    b = mock_factory.pin(2)
    with Motor(1, 2) as motor:
        motor.value = -1
        assert motor.is_active
        assert motor.value == -1
        assert b.state == 1 and f.state == 0
        motor.value = 1
        assert motor.is_active
        assert motor.value == 1
        assert b.state == 0 and f.state == 1
        motor.value = 0.5
        assert motor.is_active
        assert motor.value == 0.5
        assert b.state == 0 and f.state == 0.5
        motor.value = -0.5
        assert motor.is_active
        assert motor.value == -0.5
        assert b.state == 0.5 and f.state == 0
        motor.value = 0
        assert not motor.is_active
        assert not motor.value
        assert b.state == 0 and f.state == 0

def test_motor_value_nonpwm(mock_factory):
    f = mock_factory.pin(1)
    b = mock_factory.pin(2)
    with Motor(1, 2, pwm=False) as motor:
        motor.value = -1
        assert motor.is_active
        assert motor.value == -1
        assert b.state == 1 and f.state == 0
        motor.value = 1
        assert motor.is_active
        assert motor.value == 1
        assert b.state == 0 and f.state == 1
        motor.value = 0
        assert not motor.is_active
        assert not motor.value
        assert b.state == 0 and f.state == 0

def test_motor_bad_value(mock_factory, pwm):
    f = mock_factory.pin(1)
    b = mock_factory.pin(2)
    with Motor(1, 2) as motor:
        with pytest.raises(ValueError):
            motor.value = -2
        with pytest.raises(ValueError):
            motor.value = 2
        with pytest.raises(ValueError):
            motor.forward(2)
        with pytest.raises(ValueError):
            motor.backward(2)
        with pytest.raises(ValueError):
            motor.forward(-1)
        with pytest.raises(ValueError):
            motor.backward(-1)

def test_motor_bad_value_nonpwm(mock_factory):
    f = mock_factory.pin(1)
    b = mock_factory.pin(2)
    with Motor(1, 2, pwm=False) as motor:
        with pytest.raises(ValueError):
            motor.value = -2
        with pytest.raises(ValueError):
            motor.value = 2
        with pytest.raises(ValueError):
            motor.value = 0.5
        with pytest.raises(ValueError):
            motor.value = -0.5
        with pytest.raises(ValueError):
            motor.forward(0.5)
        with pytest.raises(ValueError):
            motor.backward(0.5)

def test_motor_reverse(mock_factory, pwm):
    f = mock_factory.pin(1)
    b = mock_factory.pin(2)
    with Motor(1, 2) as motor:
        motor.forward()
        assert motor.value == 1
        assert b.state == 0 and f.state == 1
        motor.reverse()
        assert motor.value == -1
        assert b.state == 1 and f.state == 0
        motor.backward(0.5)
        assert motor.value == -0.5
        assert b.state == 0.5 and f.state == 0
        motor.reverse()
        assert motor.value == 0.5
        assert b.state == 0 and f.state == 0.5

def test_motor_reverse_nonpwm(mock_factory):
    f = mock_factory.pin(1)
    b = mock_factory.pin(2)
    with Motor(1, 2, pwm=False) as motor:
        motor.forward()
        assert motor.value == 1
        assert b.state == 0 and f.state == 1
        motor.reverse()
        assert motor.value == -1
        assert b.state == 1 and f.state == 0

def test_motor_enable_pin_bad_init(mock_factory, pwm):
    with pytest.raises(GPIOPinMissing):
        Motor(enable=1)
    with pytest.raises(GPIOPinMissing):
        Motor(forward=1, enable=2)
    with pytest.raises(GPIOPinMissing):
        Motor(backward=1, enable=2)
    with pytest.raises(GPIOPinMissing):
        Motor(backward=1, enable=2, pwm=True)

def test_motor_enable_pin_init(mock_factory, pwm):
    f = mock_factory.pin(1)
    b = mock_factory.pin(2)
    e = mock_factory.pin(3)
    with Motor(forward=1, backward=2, enable=3) as motor:
        assert motor.forward_device.pin is f
        assert isinstance(motor.forward_device, PWMOutputDevice)
        assert motor.backward_device.pin is b
        assert isinstance(motor.backward_device, PWMOutputDevice)
        assert motor.enable_device.pin is e
        assert isinstance(motor.enable_device, DigitalOutputDevice)
        assert e.state
    with Motor(1, 2, 3) as motor:
        assert motor.forward_device.pin is f
        assert isinstance(motor.forward_device, PWMOutputDevice)
        assert motor.backward_device.pin is b
        assert isinstance(motor.backward_device, PWMOutputDevice)
        assert motor.enable_device.pin is e
        assert isinstance(motor.enable_device, DigitalOutputDevice)
        assert e.state

def test_motor_enable_pin_nonpwm_init(mock_factory):
    f = mock_factory.pin(1)
    b = mock_factory.pin(2)
    e = mock_factory.pin(3)
    with Motor(forward=1, backward=2, enable=3, pwm=False) as motor:
        assert motor.forward_device.pin is f
        assert isinstance(motor.forward_device, DigitalOutputDevice)
        assert motor.backward_device.pin is b
        assert isinstance(motor.backward_device, DigitalOutputDevice)
        assert motor.enable_device.pin is e
        assert isinstance(motor.enable_device, DigitalOutputDevice)

def test_motor_enable_pin(mock_factory, pwm):
    with Motor(forward=1, backward=2, enable=3) as motor:
        motor.forward()
        assert motor.value == 1
        motor.backward()
        assert motor.value == -1
        motor.stop()
        assert motor.value == 0

def test_phaseenable_motor_bad_init(mock_factory):
    with pytest.raises(GPIOPinMissing):
        PhaseEnableMotor()
    with pytest.raises(GPIOPinMissing):
        PhaseEnableMotor(2)
    with pytest.raises(GPIOPinMissing):
        PhaseEnableMotor(phase=2)
    with pytest.raises(GPIOPinMissing):
        PhaseEnableMotor(enable=2)
    with pytest.raises(TypeError):
        PhaseEnableMotor(a=2, b=3)

def test_phaseenable_motor_pins(mock_factory, pwm):
    p = mock_factory.pin(1)
    e = mock_factory.pin(2)
    with PhaseEnableMotor(1, 2) as motor:
        assert repr(motor).startswith('<gpiozero.PhaseEnableMotor object')
        assert motor.phase_device.pin is p
        assert isinstance(motor.phase_device, OutputDevice)
        assert motor.enable_device.pin is e
        assert isinstance(motor.enable_device, PWMOutputDevice)

def test_phaseenable_motor_pins_nonpwm(mock_factory):
    p = mock_factory.pin(1)
    e = mock_factory.pin(2)
    with PhaseEnableMotor(1, 2, pwm=False) as motor:
        assert motor.phase_device.pin is p
        assert isinstance(motor.phase_device, OutputDevice)
        assert motor.enable_device.pin is e
        assert isinstance(motor.enable_device, DigitalOutputDevice)

def test_phaseenable_motor_close(mock_factory, pwm):
    p = mock_factory.pin(1)
    e = mock_factory.pin(2)
    with PhaseEnableMotor(1, 2) as motor:
        motor.close()
        assert motor.closed
        assert motor.phase_device.pin is None
        assert motor.enable_device.pin is None
        motor.close()
        assert motor.closed

def test_phaseenable_motor_close_nonpwm(mock_factory):
    p = mock_factory.pin(1)
    e = mock_factory.pin(2)
    with PhaseEnableMotor(1, 2, pwm=False) as motor:
        motor.close()
        assert motor.closed
        assert motor.phase_device.pin is None
        assert motor.enable_device.pin is None

def test_phaseenable_motor_value(mock_factory, pwm):
    p = mock_factory.pin(1)
    e = mock_factory.pin(2)
    with PhaseEnableMotor(1, 2) as motor:
        motor.value = -1
        assert motor.is_active
        assert motor.value == -1
        assert p.state == 1 and e.state == 1
        motor.value = 1
        assert motor.is_active
        assert motor.value == 1
        assert p.state == 0 and e.state == 1
        motor.value = 0.5
        assert motor.is_active
        assert motor.value == 0.5
        assert p.state == 0 and e.state == 0.5
        motor.value = -0.5
        assert motor.is_active
        assert motor.value == -0.5
        assert p.state == 1 and e.state == 0.5
        motor.value = 0
        assert not motor.is_active
        assert not motor.value
        assert e.state == 0

def test_phaseenable_motor_value_nonpwm(mock_factory):
    p = mock_factory.pin(1)
    e = mock_factory.pin(2)
    with PhaseEnableMotor(1, 2, pwm=False) as motor:
        motor.value = -1
        assert motor.is_active
        assert motor.value == -1
        assert p.state == 1 and e.state == 1
        motor.value = 1
        assert motor.is_active
        assert motor.value == 1
        assert p.state == 0 and e.state == 1
        motor.value = 0
        assert not motor.is_active
        assert not motor.value
        assert e.state == 0

def test_phaseenable_motor_bad_value(mock_factory, pwm):
    p = mock_factory.pin(1)
    e = mock_factory.pin(2)
    with PhaseEnableMotor(1, 2) as motor:
        with pytest.raises(ValueError):
            motor.value = -2
        with pytest.raises(ValueError):
            motor.value = 2
        with pytest.raises(ValueError):
            motor.forward(2)
        with pytest.raises(ValueError):
            motor.backward(2)

def test_phaseenable_motor_bad_value_nonpwm(mock_factory):
    p = mock_factory.pin(1)
    e = mock_factory.pin(2)
    with PhaseEnableMotor(1, 2, pwm=False) as motor:
        with pytest.raises(ValueError):
            motor.value = -2
        with pytest.raises(ValueError):
            motor.value = 2
        with pytest.raises(ValueError):
            motor.value = 0.5
        with pytest.raises(ValueError):
            motor.value = -0.5

def test_phaseenable_motor_reverse(mock_factory, pwm):
    p = mock_factory.pin(1)
    e = mock_factory.pin(2)
    with PhaseEnableMotor(1, 2) as motor:
        motor.forward()
        assert motor.value == 1
        assert p.state == 0 and e.state == 1
        motor.reverse()
        assert motor.value == -1
        assert p.state == 1 and e.state == 1
        motor.backward(0.5)
        assert motor.value == -0.5
        assert p.state == 1 and e.state == 0.5
        motor.reverse()
        assert motor.value == 0.5
        assert p.state == 0 and e.state == 0.5

def test_phaseenable_motor_reverse_nonpwm(mock_factory):
    p = mock_factory.pin(1)
    e = mock_factory.pin(2)
    with PhaseEnableMotor(1, 2, pwm=False) as motor:
        motor.forward()
        assert motor.value == 1
        assert p.state == 0 and e.state == 1
        motor.reverse()
        assert motor.value == -1
        assert p.state == 1 and e.state == 1

def test_servo_pins(mock_factory, pwm):
    p = mock_factory.pin(1)
    with Servo(1) as servo:
        assert repr(servo).startswith('<gpiozero.Servo object')
        assert servo.pwm_device.pin is p
        assert isinstance(servo.pwm_device, PWMOutputDevice)

def test_servo_bad_value(mock_factory, pwm):
    p = mock_factory.pin(1)
    with pytest.raises(ValueError):
        Servo(1, initial_value=2)
    with pytest.raises(ValueError):
        Servo(1, min_pulse_width=30/1000)
    with pytest.raises(ValueError):
        Servo(1, max_pulse_width=30/1000)

def test_servo_pins_nonpwm(mock_factory):
    p = mock_factory.pin(2)
    with pytest.raises(PinPWMUnsupported):
        Servo(1)

def test_servo_close(mock_factory, pwm):
    p = mock_factory.pin(2)
    with Servo(2) as servo:
        servo.close()
        assert servo.closed
        assert servo.pwm_device.pin is None
        servo.close()
        assert servo.closed

def test_servo_pulse_width(mock_factory, pwm):
    p = mock_factory.pin(2)
    with Servo(2, min_pulse_width=5/10000, max_pulse_width=25/10000) as servo:
        assert isclose(servo.min_pulse_width, 5/10000)
        assert isclose(servo.max_pulse_width, 25/10000)
        assert isclose(servo.frame_width, 20/1000)
        assert isclose(servo.pulse_width, 15/10000)
        servo.value = -1
        assert isclose(servo.pulse_width, 5/10000)
        servo.value = 1
        assert isclose(servo.pulse_width, 25/10000)
        servo.pulse_width = 20/10000
        assert isclose(servo.pulse_width, 20/10000)
        servo.value = None
        assert servo.pulse_width is None

def test_servo_initial_values(mock_factory, pwm):
    p = mock_factory.pin(2)
    with Servo(2) as servo:
        assert servo.value == 0
    with Servo(2, initial_value=-1) as servo:
        assert servo.is_active
        assert servo.value == -1
        assert isclose(p.state, 0.05)
    with Servo(2, initial_value=0) as servo:
        assert servo.is_active
        assert servo.value == 0
        assert isclose(p.state, 0.075)
    with Servo(2, initial_value=1) as servo:
        assert servo.is_active
        assert servo.value == 1
        assert isclose(p.state, 0.1)
    with Servo(2, initial_value=None) as servo:
        assert not servo.is_active
        assert servo.value is None

def test_servo_values(mock_factory, pwm):
    p = mock_factory.pin(1)
    with Servo(1) as servo:
        servo.min()
        assert servo.is_active
        assert servo.value == -1
        assert isclose(p.state, 0.05)
        servo.max()
        assert servo.is_active
        assert servo.value == 1
        assert isclose(p.state, 0.1)
        servo.mid()
        assert servo.is_active
        assert servo.value == 0.0
        assert isclose(p.state, 0.075)
        servo.value = 0.5
        assert servo.is_active
        assert servo.value == 0.5
        assert isclose(p.state, 0.0875)
        servo.detach()
        assert not servo.is_active
        assert servo.value is None
        servo.value = 0
        assert servo.value == 0
        servo.value = None
        assert servo.value is None

def test_angular_servo_bad_value(mock_factory, pwm):
    p = mock_factory.pin(1)
    with pytest.raises(ValueError):
        AngularServo(1, initial_angle=-100)
    with pytest.raises(ValueError):
        AngularServo(1, min_angle=95)

def test_angular_servo_range(mock_factory, pwm):
    with AngularServo(1, initial_angle=15, min_angle=0, max_angle=90) as servo:
        assert repr(servo).startswith('<gpiozero.AngularServo object')
        assert servo.min_angle == 0
        assert servo.max_angle == 90

def test_angular_servo_initial_angles(mock_factory, pwm):
    with AngularServo(1) as servo:
        assert servo.angle == 0
    with AngularServo(1, initial_angle=-90) as servo:
        assert servo.angle == -90
        assert isclose(servo.value, -1)
    with AngularServo(1, initial_angle=0) as servo:
        assert servo.angle == 0
        assert isclose(servo.value, 0)
    with AngularServo(1, initial_angle=90) as servo:
        assert servo.angle == 90
        assert isclose(servo.value, 1)
    with AngularServo(1, initial_angle=None) as servo:
        assert servo.angle is None

def test_angular_servo_angles(mock_factory, pwm):
    with AngularServo(1) as servo:
        servo.angle = 0
        assert servo.angle == 0
        assert isclose(servo.value, 0)
        servo.max()
        assert servo.angle == 90
        assert isclose(servo.value, 1)
        servo.min()
        assert servo.angle == -90
        assert isclose(servo.value, -1)
        servo.detach()
        assert servo.angle is None

    with AngularServo(1, initial_angle=15, min_angle=0, max_angle=90) as servo:
        assert servo.angle == 15
        assert isclose(servo.value, -2/3)
        servo.angle = 0
        assert servo.angle == 0
        assert isclose(servo.value, -1)
        servo.angle = 90
        assert servo.angle == 90
        assert isclose(servo.value, 1)
        servo.angle = None
        assert servo.angle is None

    with AngularServo(1, min_angle=45, max_angle=-45) as servo:
        assert servo.angle == 0
        assert isclose(servo.value, 0)
        servo.angle = -45
        assert servo.angle == -45
        assert isclose(servo.value, 1)
        servo.angle = -15
        assert servo.angle == -15
        assert isclose(servo.value, 1/3)
        with pytest.raises(ValueError):
            servo.angle = 90

def test_tonalbuzzer_bad_init(mock_factory, pwm):
    with pytest.raises(ValueError):
        TonalBuzzer(2, initial_value=-2)
    with pytest.raises(ValueError):
        TonalBuzzer(2, initial_value=2)
    with pytest.raises(ValueError):
        TonalBuzzer(2, mid_tone='foo')
    with pytest.raises(ValueError):
        TonalBuzzer(2, octaves=0)
    with pytest.raises(ValueError):
        TonalBuzzer(2, octaves=0)
    with pytest.raises(ValueError):
        TonalBuzzer(2, mid_tone='B0', octaves=2)
    with pytest.raises(ValueError):
        TonalBuzzer(2, mid_tone='B1', octaves=3)
    with pytest.raises(ValueError):
        TonalBuzzer(2, mid_tone='B2', octaves=4)
    with pytest.raises(ValueError):
        TonalBuzzer(2, mid_tone='C6', octaves=4)

def test_tonalbuzzer_init(mock_factory, pwm):
    pin = mock_factory.pin(2)
    with TonalBuzzer(2) as tb:
        assert repr(tb).startswith('<gpiozero.TonalBuzzer object')
        assert tb.pwm_device.pin is pin
        assert tb.value is None
        assert tb.pwm_device.frequency is None
        tb.play('C5')
        assert 'playing' in repr(tb)
        assert tb.is_active
        tb.stop()
        assert not tb.is_active
    assert repr(tb) == '<gpiozero.TonalBuzzer object closed>'
    with TonalBuzzer(2, mid_tone='C4') as tb:
        assert tb.pwm_device.frequency is None
    with TonalBuzzer(2, mid_tone='C4', initial_value=0) as tb:
        assert isclose(tb.pwm_device.frequency, 261.626, abs_tol=1/100)
    with TonalBuzzer(2, initial_value=-1) as tb:
        assert isclose(tb.pwm_device.frequency, 220)
    with TonalBuzzer(2, initial_value=0) as tb:
        assert isclose(tb.pwm_device.frequency, 440)
    with TonalBuzzer(2, initial_value=1) as tb:
        assert isclose(tb.pwm_device.frequency, 880)
    with TonalBuzzer(2, octaves=2, initial_value=-1) as tb:
        assert isclose(tb.pwm_device.frequency, 110)
    with TonalBuzzer(2, octaves=2, initial_value=0) as tb:
        assert isclose(tb.pwm_device.frequency, 440)
    with TonalBuzzer(2, octaves=2, initial_value=1) as tb:
        assert isclose(tb.pwm_device.frequency, 1760)

def test_tonalbuzzer_play(mock_factory, pwm):
    with TonalBuzzer(2) as tb:
        tb.play(60)
        assert isclose(tb.pwm_device.frequency, 261.626, abs_tol=1/100)
        tb.play(None)
        assert tb.value is None
        assert tb.pwm_device.frequency is None
        tb.play('C5')
        assert isclose(tb.pwm_device.frequency, 523.25, abs_tol=1/100)
        tb.play(Tone('A#4'))
        assert isclose(tb.pwm_device.frequency, 466.16, abs_tol=1/100)
        tb.stop()
        assert tb.value is None
        assert tb.pwm_device.frequency is None
        with pytest.raises(ValueError):
            tb.play('G#3')
        with pytest.raises(ValueError):
            tb.play('A#5')

def test_tonalbuzzer_set_value(mock_factory, pwm):
    with TonalBuzzer(2) as tb:
        assert tb.pwm_device.frequency is None
        tb.value = -1
        assert isclose(tb.pwm_device.frequency, 220)
        tb.value = 1
        assert isclose(tb.pwm_device.frequency, 880)
    with TonalBuzzer(2, octaves=2) as tb:
        assert tb.pwm_device.frequency is None
        tb.value = -1
        assert isclose(tb.pwm_device.frequency, 110)
        tb.value = 1
        assert isclose(tb.pwm_device.frequency, 1760)

def test_tonalbuzzer_read_value(mock_factory, pwm):
    with TonalBuzzer(2) as tb:
        assert tb.value is None
        tb.play('A3')
        assert isclose(tb.value, -1)
        tb.play('A4')
        assert isclose(tb.value, 0)
        tb.play('A5')
        assert isclose(tb.value, 1)
    with TonalBuzzer(2, octaves=2) as tb:
        assert tb.value is None
        tb.play('A2')
        assert isclose(tb.value, -1)
        tb.play('A3')
        assert isclose(tb.value, -0.5)
        tb.play('A4')
        assert isclose(tb.value, 0)
        tb.play('A5')
        assert isclose(tb.value, 0.5)
        tb.play('A6')
        assert isclose(tb.value, 1)

def test_tonalbuzzer_tone(mock_factory, pwm):
    with TonalBuzzer(2) as tb:
        assert tb.tone is None
        tb.play('A3')
        assert tb.tone == Tone('A3')
        tb.play('A4')
        assert tb.tone == Tone('A4')
        tb.play('A5')
        assert tb.tone == Tone('A5')
        tb.tone = None
        assert tb.tone is None



from gpiozero import LEDBoard
from time import sleep

leds = LEDBoard(2, 3, 4, 5, 6, 7, 8, 9)

for led in leds[3:]:  # leds 3 and onward
    led.on()
sleep(1)
leds.off()

for led in leds[:2]:  # leds 0 and 1
    led.on()
sleep(1)
leds.off()

for led in leds[::2]:  # even leds (0, 2, 4...)
    led.on()
sleep(1)
leds.off()

for led in leds[1::2]:  # odd leds (1, 3, 5...)
    led.on()
sleep(1)
leds.off()



# vim: set fileencoding=utf-8:
#
# GPIO Zero: a library for controlling the Raspberry Pi's GPIO pins
#
# Copyright (c) 2016-2021 Dave Jones <dave@waveform.org.uk>
# Copyright (c) 2019 Ben Nuttall <ben@bennuttall.com>
# Copyright (c) 2018 Martchus <martchus@gmx.net>
#
# SPDX-License-Identifier: BSD-3-Clause

from __future__ import (
    unicode_literals,
    absolute_import,
    print_function,
    division,
    )
str = type('')


import io
import re
import errno
import pytest
from mock import patch, MagicMock

import gpiozero.pins.data
import gpiozero.pins.local
from gpiozero.pins.local import LocalPiFactory
from gpiozero.pins.data import Style, HeaderInfo, PinInfo
from gpiozero import *


def test_pi_revision():
    with patch('gpiozero.devices.Device.pin_factory', LocalPiFactory()):
        # Can't use MockPin for this as we want something that'll actually try
        # and read /proc/device-tree/system/linux,revision and /proc/cpuinfo
        # (MockPin simply parrots the 3B's data); LocalPiFactory is used as we
        # can definitely instantiate it (strictly speaking it's abstract but
        # we're only interested in the pi_info stuff)
        with patch('io.open') as m:
            m.return_value.__enter__.side_effect = [
                # Pretend /proc/device-tree/system/linux,revision doesn't
                # exist, and that /proc/cpuinfo contains the Revision: 0002
                # after some filler
                IOError(errno.ENOENT, 'File not found'),
                ['lots of irrelevant', 'lines', 'followed by', 'Revision: 0002', 'Serial:  xxxxxxxxxxx']
            ]
            assert pi_info().revision == '0002'
            # LocalPiFactory caches the revision (because realistically it
            # isn't going to change at runtime); we need to wipe it here though
            Device.pin_factory._info = None
            m.return_value.__enter__.side_effect = [
                IOError(errno.ENOENT, 'File not found'),
                ['Revision: a21042']
            ]
            assert pi_info().revision == 'a21042'
            # Check over-volting result (some argument over whether this is 7
            # or 8 character result; make sure both work)
            Device.pin_factory._info = None
            m.return_value.__enter__.side_effect = [
                IOError(errno.ENOENT, 'File not found'),
                ['Revision: 1000003']
            ]
            assert pi_info().revision == '0003'
            Device.pin_factory._info = None
            m.return_value.__enter__.side_effect = [
                IOError(errno.ENOENT, 'File not found'),
                ['Revision: 100003']
            ]
            assert pi_info().revision == '0003'
            # Check we complain loudly if we can't access linux,revision
            Device.pin_factory._info = None
            m.return_value.__enter__.side_effect = [
                # Pretend /proc/device-tree/system/linux,revision doesn't
                # exist, and that /proc/cpuinfo contains the Revision: 0002
                # after some filler
                IOError(errno.EACCES, 'Permission denied'),
                ['Revision: 100003']
            ]
            with pytest.raises(IOError):
                pi_info()
            # Check that parsing /proc/device-tree/system/linux,revision also
            # works properly
            Device.pin_factory._info = None
            m.return_value.__enter__.side_effect = None
            m.return_value.__enter__.return_value = io.BytesIO(b'\x00\xa2\x20\xd3')
            assert pi_info().revision == 'a220d3'
            # Check that if everything's a bust we raise PinUnknownPi
            with pytest.raises(PinUnknownPi):
                Device.pin_factory._info = None
                m.return_value.__enter__.return_value = None
                m.return_value.__enter__.side_effect = [
                    IOError(errno.ENOENT, 'File not found'),
                    ['nothing', 'relevant']
                ]
                pi_info()
            with pytest.raises(PinUnknownPi):
                pi_info('0fff')

def test_pi_info():
    r = pi_info('900011')
    assert r.model == 'B'
    assert r.pcb_revision == '1.0'
    assert r.memory == 512
    assert r.manufacturer == 'Sony'
    assert r.storage == 'SD'
    assert r.usb == 2
    assert r.ethernet == 1
    assert not r.wifi
    assert not r.bluetooth
    assert r.csi == 1
    assert r.dsi == 1
    r = pi_info('9000f1')
    assert r.model == '???'
    assert r.pcb_revision == '1.1'
    assert r.memory == 512
    assert r.manufacturer == 'Sony'
    assert r.storage == 'MicroSD'
    assert r.usb == 4
    assert r.ethernet == 1
    assert not r.wifi
    assert not r.bluetooth
    assert r.csi == 1
    assert r.dsi == 1
    assert repr(r).startswith('PiBoardInfo(revision=')
    assert 'headers=...' in repr(r)

def test_pi_info_not_a_pi():
    class NotAPiFactory(LocalPiFactory):
        def _get_pi_info(self):
            return None
    with patch('gpiozero.devices.Device.pin_factory', NotAPiFactory()):
        with pytest.raises(PinUnknownPi):
            pi_info()

def test_pi_info_other_types():
    assert pi_info(b'9000f1') == pi_info(0x9000f1)

def test_physical_pins():
    # Assert physical pins for some well-known Pi's; a21041 is a Pi2B
    assert pi_info('a21041').physical_pins('3V3') == {('J8', 1), ('J8', 17)}
    assert pi_info('a21041').physical_pins('GPIO2') == {('J8', 3)}
    assert pi_info('a21041').physical_pins('GPIO47') == set()

def test_physical_pin():
    with pytest.raises(PinMultiplePins):
        assert pi_info('a21041').physical_pin('GND')
    assert pi_info('a21041').physical_pin('GPIO3') == ('J8', 5)
    with pytest.raises(PinNoPins):
        assert pi_info('a21041').physical_pin('GPIO47')

def test_pulled_up():
    assert pi_info('a21041').pulled_up('GPIO2')
    assert not pi_info('a21041').pulled_up('GPIO4')
    assert not pi_info('a21041').pulled_up('GPIO47')

def test_pprint_content():
    with patch('sys.stdout') as stdout:
        stdout.output = []
        stdout.write = lambda buf: stdout.output.append(buf)
        pi_info('900092').pprint(color=False)
        s = ''.join(stdout.output)
        assert ('o' * 20 + ' ') in s # first header row
        assert ('1' + 'o' * 19 + ' ') in s # second header row
        assert 'PiZero' in s
        assert 'V1.2' in s # PCB revision
        assert '900092' in s # Pi revision
        assert 'BCM2835' in s # SOC name
        stdout.output = []
        pi_info('0002').pprint(color=False)
        s = ''.join(stdout.output)
        assert ('o' * 13 + ' ') in s # first header row
        assert ('1' + 'o' * 12 + ' ') in s # second header row
        assert 'Pi Model' in s
        assert 'B  V1.0' in s # PCB revision
        assert '0002' in s # Pi revision
        assert 'BCM2835' in s # SOC name
        stdout.output = []
        pi_info('0014').headers['SODIMM'].pprint(color=False)
        assert len(''.join(stdout.output).splitlines()) == 100

def test_format_content():
    with patch('sys.stdout') as stdout:
        stdout.output = []
        stdout.write = lambda buf: stdout.output.append(buf)
        pi_info('900092').pprint(color=False)
        s = ''.join(stdout.output)
        assert '{0:mono}\n'.format(pi_info('900092')) == s
        stdout.output = []
        pi_info('900092').pprint(color=True)
        s = ''.join(stdout.output)
        assert '{0:color full}\n'.format(pi_info('900092')) == s
        with pytest.raises(ValueError):
            '{0:color foo}'.format(pi_info('900092'))

def test_pprint_headers():
    assert len(pi_info('0002').headers) == 1
    assert len(pi_info('000e').headers) == 2
    assert len(pi_info('900092').headers) == 1
    with patch('sys.stdout') as stdout:
        stdout.output = []
        stdout.write = lambda buf: stdout.output.append(buf)
        pi_info('0002').pprint()
        s = ''.join(stdout.output)
        assert 'P1:\n' in s
        assert 'P5:\n' not in s
        stdout.output = []
        pi_info('000e').pprint()
        s = ''.join(stdout.output)
        assert 'P1:\n' in s
        assert 'P5:\n' in s
        stdout.output = []
        pi_info('900092').pprint()
        s = ''.join(stdout.output)
        assert 'J8:\n' in s
        assert 'P1:\n' not in s
        assert 'P5:\n' not in s

def test_format_headers():
    with patch('sys.stdout') as stdout:
        stdout.output = []
        stdout.write = lambda buf: stdout.output.append(buf)
        info = pi_info('c03131')
        info.headers['J8'].pprint(color=False)
        s = ''.join(stdout.output)
        assert '{0.headers[J8]:mono}\n'.format(info) == s
        stdout.output = []
        info.headers['J8'].pprint(color=True)
        s = ''.join(stdout.output)
        assert '{0.headers[J8]:color}\n'.format(info) == s
        with pytest.raises(ValueError):
            '{0.headers[J8]:mono foo}'.format(info)

def test_pprint_color():
    with patch('sys.stdout') as stdout:
        stdout.output = []
        stdout.write = lambda buf: stdout.output.append(buf)
        pi_info('900092').pprint(color=False)
        s = ''.join(stdout.output)
        assert '\x1b[0m' not in s # make sure ANSI reset code isn't in there
        stdout.output = []
        pi_info('900092').pprint(color=True)
        s = ''.join(stdout.output)
        assert '\x1b[0m' in s # check the ANSI reset code *is* in there (can't guarantee much else!)
        stdout.output = []
        stdout.fileno.side_effect = IOError('not a real file')
        pi_info('900092').pprint()
        s = ''.join(stdout.output)
        assert '\x1b[0m' not in s # default should output mono
        with patch('os.isatty') as isatty:
            isatty.return_value = True
            stdout.fileno.side_effect = None
            stdout.output = []
            pi_info('900092').pprint()
            s = ''.join(stdout.output)
            assert '\x1b[0m' in s # default should now output color

def test_pprint_styles():
    with pytest.raises(ValueError):
        Style.from_style_content('mono color full')
    with pytest.raises(ValueError):
        Style.from_style_content('full specs')
    with patch('sys.stdout') as stdout:
        s = '{0:full}'.format(pi_info('900092'))
        assert '\x1b[0m' not in s # ensure default is mono when stdout is not a tty
    with pytest.raises(ValueError):
        '{0:foo on bar}'.format(Style())

def test_pprint_missing_pin():
    header = HeaderInfo('FOO', 4, 2, {
        1: PinInfo(1, '5V',    False, 1, 1),
        2: PinInfo(2, 'GND',   False, 1, 2),
        # Pin 3 is deliberately missing
        4: PinInfo(4, 'GPIO1', False, 2, 2),
        5: PinInfo(5, 'GPIO2', False, 3, 1),
        6: PinInfo(6, 'GPIO3', False, 3, 2),
        7: PinInfo(7, '3V3',   False, 4, 1),
        8: PinInfo(8, 'GND',   False, 4, 2),
        })
    with patch('sys.stdout') as stdout:
        stdout.output = []
        stdout.write = lambda buf: stdout.output.append(buf)
        s = ''.join(stdout.output)
        header.pprint()
        for i in range(1, 9):
            if i == 3:
                assert '(3)' not in s
            else:
                assert ('(%d)' % i)

def test_pprint_rows_cols():
    assert '{0:row1}'.format(pi_info('900092').headers['J8']) == '1o'
    assert '{0:row2}'.format(pi_info('900092').headers['J8']) == 'oo'
    assert '{0:col1}'.format(pi_info('0002').headers['P1']) == '1oooooooooooo'
    assert '{0:col2}'.format(pi_info('0002').headers['P1']) == 'ooooooooooooo'
    with pytest.raises(ValueError):
        '{0:row16}'.format(pi_info('0002').headers['P1'])
    with pytest.raises(ValueError):
        '{0:col3}'.format(pi_info('0002').headers['P1'])



from gpiozero import LED
from gpiozero.pins.pigpio import PiGPIOFactory
from time import sleep

factory3 = PiGPIOFactory(host='192.168.1.3')
factory4 = PiGPIOFactory(host='192.168.1.4')

led_1 = LED(17)  # local pin
led_2 = LED(17, pin_factory=factory3)  # remote pin on one pi
led_3 = LED(17, pin_factory=factory4)  # remote pin on another pi

while True:
    led_1.on()
    led_2.off()
    led_3.on()
    sleep(1)
    led_1.off()
    led_2.on()
    led_3.off()
    sleep(1)



#!/usr/bin/env python3
# vim: set fileencoding=utf-8:
#
# GPIO Zero: a library for controlling the Raspberry Pi's GPIO pins
#
# Copyright (c) 2015-2021 Dave Jones <dave@waveform.org.uk>
# Copyright (c) 2019-2020 Ben Nuttall <ben@bennuttall.com>
# Copyright (c) 2016 Thijs Triemstra <info@collab.nl>
#
# SPDX-License-Identifier: BSD-3-Clause

import sys
import os
from datetime import datetime
sys.path.insert(0, os.path.join(os.path.dirname(__file__), '..'))
on_rtd = os.environ.get('READTHEDOCS', None) == 'True'
import setup as _setup

# Mock out certain modules while building documentation
class Mock(object):
    __all__ = []

    def __init__(self, *args, **kw):
        pass

    def __call__(self, *args, **kw):
        return Mock()

    def __mul__(self, other):
        return Mock()

    def __and__(self, other):
        return Mock()

    def __bool__(self):
        return False

    def __nonzero__(self):
        return False

    @classmethod
    def __getattr__(cls, name):
        if name in ('__file__', '__path__'):
            return '/dev/null'
        else:
            return Mock()

sys.modules['RPi'] = Mock()
sys.modules['RPi.GPIO'] = sys.modules['RPi'].GPIO
sys.modules['lgpio'] = Mock()
sys.modules['RPIO'] = Mock()
sys.modules['RPIO.PWM'] = sys.modules['RPIO'].PWM
sys.modules['RPIO.Exceptions'] = sys.modules['RPIO'].Exceptions
sys.modules['pigpio'] = Mock()
sys.modules['w1thermsensor'] = Mock()
sys.modules['spidev'] = Mock()
sys.modules['colorzero'] = Mock()

# -- General configuration ------------------------------------------------

extensions = ['sphinx.ext.autodoc', 'sphinx.ext.viewcode', 'sphinx.ext.intersphinx']
templates_path = ['_templates']
source_suffix = '.rst'
#source_encoding = 'utf-8-sig'
master_doc = 'index'
project = 'GPIO Zero'
copyright = '2015-%s %s' % (datetime.now().year, _setup.__author__)
version = _setup.__version__
release = _setup.__version__
#language = None
#today_fmt = '%B %d, %Y'
exclude_patterns = ['_build']
highlight_language='python3'
#default_role = None
#add_function_parentheses = True
#add_module_names = True
#show_authors = False
pygments_style = 'sphinx'
#modindex_common_prefix = []
#keep_warnings = False

# -- Autodoc configuration ------------------------------------------------

autodoc_member_order = 'groupwise'

# -- Intersphinx configuration --------------------------------------------

intersphinx_mapping = {
    'python': ('https://docs.python.org/3.7', None),
    'picamera': ('https://picamera.readthedocs.io/en/latest', None),
    'colorzero': ('https://colorzero.readthedocs.io/en/latest', None),
    }

# -- Options for HTML output ----------------------------------------------

html_theme = 'sphinx_rtd_theme'
html_title = '%s %s Documentation' % (project, version)
#html_theme_path = []
#html_short_title = None
#html_logo = None
#html_favicon = None
html_static_path = ['_static']
#html_extra_path = []
#html_last_updated_fmt = '%b %d, %Y'
#html_use_smartypants = True
#html_additional_pages = {}
#html_domain_indices = True
#html_use_index = True
#html_split_index = False
#html_show_sourcelink = True
#html_show_sphinx = True
#html_show_copyright = True
#html_use_opensearch = ''
#html_file_suffix = None
htmlhelp_basename = '%sdoc' % _setup.__project__

# Hack to make wide tables work properly in RTD
# See https://github.com/snide/sphinx_rtd_theme/issues/117 for details
#def setup(app):
#    app.add_stylesheet('style_override.css')

# -- Options for LaTeX output ---------------------------------------------

latex_engine = 'xelatex'

latex_elements = {
    'papersize': 'a4paper',
    'pointsize': '10pt',
    'preamble': r'\def\thempfootnote{\arabic{mpfootnote}}', # workaround sphinx issue #2530
}

latex_documents = [
    (
        'index',                       # source start file
        '%s.tex' % _setup.__project__, # target filename
        '%s Documentation' % project,  # title
        _setup.__author__,             # author
        'manual',                      # documentclass
        True,                          # documents ref'd from toctree only
        ),
]

#latex_logo = None
#latex_use_parts = False
latex_show_pagerefs = True
latex_show_urls = 'footnote'
#latex_appendices = []
#latex_domain_indices = True

# -- Options for epub output ----------------------------------------------

epub_basename = _setup.__project__
#epub_theme = 'epub'
#epub_title = html_title
epub_author = _setup.__author__
epub_identifier = 'https://gpiozero.readthedocs.io/'
#epub_tocdepth = 3
epub_show_urls = 'no'
#epub_use_index = True

# -- Options for manual page output ---------------------------------------

man_pages = [
    ('cli_pinout',  'pinout',      'GPIO Zero pinout tool',       [_setup.__author__], 1),
    ('remote_gpio', 'remote-gpio', 'GPIO Zero remote GPIO guide', [_setup.__author__], 7),
]

man_show_urls = True

# -- Options for Texinfo output -------------------------------------------

texinfo_documents = []

#texinfo_appendices = []
#texinfo_domain_indices = True
#texinfo_show_urls = 'footnote'
#texinfo_no_detailmenu = False



from gpiozero import LEDBoard, MotionSensor
from gpiozero.pins.pigpio import PiGPIOFactory
from signal import pause

ips = ['192.168.1.3', '192.168.1.4', '192.168.1.5', '192.168.1.6']
remotes = [PiGPIOFactory(host=ip) for ip in ips]

button = Button(17)  # button on this pi
buzzers = [Buzzer(pin, pin_factory=r) for r in remotes]  # buzzers on remote pins

for buzzer in buzzers:
    buzzer.source = button

pause()



# vim: set fileencoding=utf-8:
#
# GPIO Zero: a library for controlling the Raspberry Pi's GPIO pins
#
# Copyright (c) 2016-2021 Dave Jones <dave@waveform.org.uk>
#
# SPDX-License-Identifier: BSD-3-Clause

from __future__ import (
    unicode_literals,
    print_function,
    absolute_import,
    division,
    )
str = type('')


import operator
from threading import RLock

from ..devices import Device, SharedMixin
from ..input_devices import InputDevice
from ..output_devices import OutputDevice


class SPISoftwareBus(SharedMixin, Device):
    def __init__(self, clock_pin, mosi_pin, miso_pin):
        self.lock = None
        self.clock = None
        self.mosi = None
        self.miso = None
        super(SPISoftwareBus, self).__init__()
        self.lock = RLock()
        try:
            self.clock = OutputDevice(clock_pin, active_high=True)
            if mosi_pin is not None:
                self.mosi = OutputDevice(mosi_pin)
            if miso_pin is not None:
                self.miso = InputDevice(miso_pin)
        except:
            self.close()
            raise

    def close(self):
        super(SPISoftwareBus, self).close()
        if getattr(self, 'lock', None):
            with self.lock:
                if self.miso is not None:
                    self.miso.close()
                    self.miso = None
                if self.mosi is not None:
                    self.mosi.close()
                    self.mosi = None
                if self.clock is not None:
                    self.clock.close()
                    self.clock = None
        self.lock = None

    @property
    def closed(self):
        return self.lock is None

    @classmethod
    def _shared_key(cls, clock_pin, mosi_pin, miso_pin):
        return (clock_pin, mosi_pin, miso_pin)

    def transfer(self, data, clock_phase=False, lsb_first=False, bits_per_word=8):
        """
        Writes data (a list of integer words where each word is assumed to have
        :attr:`bits_per_word` bits or less) to the SPI interface, and reads an
        equivalent number of words, returning them as a list of integers.
        """
        result = []
        with self.lock:
            # See https://en.wikipedia.org/wiki/Serial_Peripheral_Interface_Bus
            # (specifically the section "Example of bit-banging the master
            # protocol") for a simpler C implementation of this which ignores
            # clock polarity, phase, variable word-size, and multiple input
            # words
            if lsb_first:
                shift = operator.lshift
                init_mask = 1
            else:
                shift = operator.rshift
                init_mask = 1 << (bits_per_word - 1)
            for write_word in data:
                mask = init_mask
                read_word = 0
                for _ in range(bits_per_word):
                    if self.mosi is not None:
                        self.mosi.value = bool(write_word & mask)
                    # read bit on clock activation
                    self.clock.on()
                    if not clock_phase:
                        if self.miso is not None and self.miso.value:
                            read_word |= mask
                    # read bit on clock deactivation
                    self.clock.off()
                    if clock_phase:
                        if self.miso is not None and self.miso.value:
                            read_word |= mask
                    mask = shift(mask, 1)
                result.append(read_word)
        return result



# vim: set fileencoding=utf-8:
#
# GPIO Zero: a library for controlling the Raspberry Pi's GPIO pins
#
# Copyright (c) 2019-2021 Dave Jones <dave@waveform.org.uk>
#
# SPDX-License-Identifier: BSD-3-Clause

from __future__ import (
    unicode_literals,
    absolute_import,
    print_function,
    division,
    )
str = type('')


import gc
import sys
from time import sleep
from threading import Event

import pytest

from gpiozero import *


def test_source_delay(mock_factory):
    with OutputDevice(2) as device:
        device.source_delay = 1
        assert device.source_delay == 1
        device.source_delay = 0.1
        assert device.source_delay == 0.1
        with pytest.raises(ValueError):
            device.source_delay = -1


def test_source(mock_factory):
    pin = mock_factory.pin(4)
    with InputDevice(4) as in_dev, OutputDevice(3) as out_dev:
        assert out_dev.source is None
        out_dev.source = in_dev.values
        assert out_dev.source is not None
        assert out_dev.value == 0
        pin.drive_high()
        # Give the output device some time to read the input device state
        sleep(0.1)
        assert out_dev.value == 1


def test_active_time(mock_factory):
    pin = mock_factory.pin(4)
    with DigitalInputDevice(4) as dev:
        assert dev.active_time is None
        assert dev.inactive_time >= 0.0
        pin.drive_high()
        sleep(0.1)
        assert dev.active_time >= 0.1
        assert dev.inactive_time is None
        pin.drive_low()
        sleep(0.1)
        assert dev.active_time is None
        assert dev.inactive_time >= 0.1


def test_basic_callbacks(mock_factory):
    pin = mock_factory.pin(4)
    evt = Event()
    with DigitalInputDevice(4) as dev:
        dev.when_activated = evt.set
        assert dev.when_activated is not None
        pin.drive_high()
        assert evt.wait(0.1)
        pin.drive_low()
        dev.when_activated = None
        assert dev.when_activated is None
        evt.clear()
        pin.drive_high()
        assert not evt.wait(0.1)


def test_builtin_callbacks(mock_factory):
    pin = mock_factory.pin(4)
    with DigitalInputDevice(4) as dev:
        assert gc.isenabled()
        dev.when_activated = gc.disable
        assert dev.when_activated is gc.disable
        pin.drive_high()
        assert not gc.isenabled()
        gc.enable()


def test_callback_with_param(mock_factory):
    pin = mock_factory.pin(4)
    with DigitalInputDevice(4) as dev:
        devices = []
        evt = Event()
        def cb(d):
            devices.append(d)
            evt.set()
        dev.when_activated = cb
        assert dev.when_activated is not None
        pin.drive_high()
        assert evt.wait(1)
        assert devices == [dev]


def test_bad_callback(mock_factory):
    pin = mock_factory.pin(4)
    with DigitalInputDevice(4) as dev:
        with pytest.raises(BadEventHandler):
            dev.when_activated = 100
        with pytest.raises(BadEventHandler):
            dev.when_activated = lambda x, y: x + y


def test_shared_key(mock_factory):
    class SharedDevice(SharedMixin, GPIODevice):
        def __init__(self, pin, pin_factory=None):
            super(SharedDevice, self).__init__(pin, pin_factory=pin_factory)

        def _conflicts_with(self, other):
            return not isinstance(other, SharedDevice)

    with SharedDevice(4) as dev:
        with SharedDevice(4) as another_dev:
            pass
        with pytest.raises(GPIOPinInUse):
            GPIODevice(4)



from gpiozero import Button
from subprocess import check_call
from signal import pause

def shutdown():
    check_call(['sudo', 'poweroff'])

shutdown_btn = Button(17, hold_time=2)
shutdown_btn.when_held = shutdown

pause()



from gpiozero import LEDBoard
from time import sleep

leds = LEDBoard(2, 3, 4, 5, 6, 7, 8, 9)

leds[0].on()  # first led on
sleep(1)
leds[7].on()  # last led on
sleep(1)
leds[-1].off()  # last led off
sleep(1)



# vim: set fileencoding=utf-8:
#
# GPIO Zero: a library for controlling the Raspberry Pi's GPIO pins
#
# Copyright (c) 2016-2021 Dave Jones <dave@waveform.org.uk>
# Copyright (c) 2019 Ben Nuttall <ben@bennuttall.com>
# Copyright (c) 2016 Andrew Scheller <github@loowis.durge.org>
#
# SPDX-License-Identifier: BSD-3-Clause

from __future__ import (
    unicode_literals,
    absolute_import,
    print_function,
    division,
    )
str = type('')

import os
import warnings
from mock import patch
import pytest
import errno

from gpiozero import *
from gpiozero.pins.mock import MockFactory


file_not_found = IOError(errno.ENOENT, 'File not found')


def test_default_pin_factory_order():
    with patch('sys.path') as path, \
         patch('io.open') as io, \
         patch('os.environ.get') as get:
        # ensure no pin libraries can be imported
        path.return_value = []
        # ensure /proc/device-tree... is not found when trying native
        io.return_value.__enter__.side_effect = file_not_found
        # ensure pin factory not set in env var
        get.return_value = None
        with warnings.catch_warnings(record=True) as ws:
            warnings.resetwarnings()
            with pytest.raises(BadPinFactory):
                device = GPIODevice(2)
            assert len(ws) == 5
            assert all(w.category == PinFactoryFallback for w in ws)
            assert ws[0].message.args[0].startswith('Falling back from rpigpio:')
            assert ws[1].message.args[0].startswith('Falling back from lgpio:')
            assert ws[2].message.args[0].startswith('Falling back from rpio:')
            assert ws[3].message.args[0].startswith('Falling back from pigpio:')
            assert ws[4].message.args[0].startswith('Falling back from native:')

def test_device_bad_pin(mock_factory):
    with pytest.raises(GPIOPinMissing):
        device = GPIODevice()
    with pytest.raises(PinInvalidPin):
        device = GPIODevice(60)
    with pytest.raises(PinInvalidPin):
        device = GPIODevice('60')
    with pytest.raises(PinInvalidPin):
        device = GPIODevice('BCM60')
    with pytest.raises(PinInvalidPin):
        device = GPIODevice('WPI32')
    with pytest.raises(PinInvalidPin):
        device = GPIODevice(b'P2:2')
    with pytest.raises(PinInvalidPin):
        device = GPIODevice('J8:42')
    with pytest.raises(PinInvalidPin):
        device = GPIODevice('J8:1')
    with pytest.raises(PinInvalidPin):
        device = GPIODevice('J8:A')
    with pytest.raises(PinInvalidPin):
        device = GPIODevice('foo')

def test_device_non_physical(mock_factory):
    with warnings.catch_warnings(record=True) as w:
        warnings.resetwarnings()
        device = GPIODevice('GPIO37')
        assert len(w) == 1
        assert w[0].category == PinNonPhysical

def test_device_init(mock_factory):
    pin = mock_factory.pin(2)
    with GPIODevice(2) as device:
        assert repr(device).startswith('<gpiozero.GPIODevice object')
        assert not device.closed
        assert device.pin is pin
    assert repr(device) == '<gpiozero.GPIODevice object closed>'
    with pytest.raises(TypeError):
        GPIODevice(2, foo='bar')

def test_device_init_twice_same_pin(mock_factory):
    with GPIODevice(2) as device:
        with pytest.raises(GPIOPinInUse):
            GPIODevice(2)

def test_device_init_twice_same_pin_different_spec(mock_factory):
    with GPIODevice(2) as device:
        with pytest.raises(GPIOPinInUse):
            GPIODevice("BOARD3")

def test_device_init_twice_different_pin(mock_factory):
    with GPIODevice(2) as device:
        with GPIODevice(3) as device2:
            pass

def test_device_close(mock_factory):
    device = GPIODevice(2)
    # Don't use "with" here; we're testing close explicitly
    device.close()
    assert device.closed
    assert device.pin is None

def test_device_reopen_same_pin(mock_factory):
    pin = mock_factory.pin(2)
    with GPIODevice(2) as device:
        pass
    with GPIODevice(2) as device2:
        assert not device2.closed
        assert device2.pin is pin
        assert device.closed
        assert device.pin is None

def test_device_pin_parsing(mock_factory):
    # MockFactory defaults to a Pi 3B layout
    pin = mock_factory.pin(2)
    with GPIODevice('2') as device:
        assert device.pin is pin
    with GPIODevice('GPIO2') as device:
        assert device.pin is pin
    with GPIODevice('BCM2') as device:
        assert device.pin is pin
    with GPIODevice('WPI8') as device:
        assert device.pin is pin
    with GPIODevice('BOARD3') as device:
        assert device.pin is pin
    with GPIODevice('J8:3') as device:
        assert device.pin is pin

def test_device_repr(mock_factory):
    with GPIODevice(4) as device:
        assert repr(device) == (
            '<gpiozero.GPIODevice object on pin %s, '
            'is_active=False>' % device.pin)

def test_device_repr_after_close(mock_factory):
    with GPIODevice(2) as device:
        pass
    assert repr(device) == '<gpiozero.GPIODevice object closed>'

def test_device_unknown_attr(mock_factory):
    with GPIODevice(2) as device:
        with pytest.raises(AttributeError):
            device.foo = 1

def test_device_broken_attr(mock_factory):
    with GPIODevice(2) as device:
        del device._active_state
        with pytest.raises(AttributeError):
            device.value

def test_device_context_manager(mock_factory):
    with GPIODevice(2) as device:
        assert not device.closed
    assert device.closed

def test_composite_device_sequence(mock_factory):
    with CompositeDevice(InputDevice(4), InputDevice(5)) as device:
        assert repr(device).startswith('<gpiozero.CompositeDevice object')
        assert len(device) == 2
        assert device[0].pin.number == 4
        assert device[1].pin.number == 5
        assert device.namedtuple._fields == ('device_0', 'device_1')

def test_composite_device_values(mock_factory):
    with CompositeDevice(InputDevice(4), InputDevice(5)) as device:
        assert repr(device) == '<gpiozero.CompositeDevice object containing 2 unnamed devices>'
        assert device.value == (0, 0)
        assert not device.is_active
        device[0].pin.drive_high()
        assert device.value == (1, 0)
        assert device.is_active

def test_composite_device_named(mock_factory):
    with CompositeDevice(
            foo=InputDevice(4),
            bar=InputDevice(5),
            _order=('foo', 'bar')
            ) as device:
        assert repr(device) == '<gpiozero.CompositeDevice object containing 2 devices: foo, bar>'
        assert device.namedtuple._fields == ('foo', 'bar')
        assert device.value == (0, 0)
        assert not device.is_active

def test_composite_device_some_named(mock_factory):
    with CompositeDevice(
            InputDevice(4),
            foobar=InputDevice(5),
            ) as device:
        assert repr(device) == '<gpiozero.CompositeDevice object containing 2 devices: foobar and 1 unnamed>'
        assert device.namedtuple._fields == ('device_0', 'foobar')
        assert device.value == (0, 0)
        assert not device.is_active

def test_composite_device_bad_init(mock_factory):
    with pytest.raises(ValueError):
        CompositeDevice(foo=1, bar=2, _order=('foo',))
    with pytest.raises(ValueError):
        CompositeDevice(close=1)
    with pytest.raises(ValueError):
        CompositeDevice(2)
    with pytest.raises(ValueError):
        CompositeDevice(mock_factory.pin(2))

def test_composite_device_read_only(mock_factory):
    with CompositeDevice(foo=InputDevice(4), bar=InputDevice(5)) as device:
        with pytest.raises(AttributeError):
            device.foo = 1

def test_shutdown(mock_factory):
    from gpiozero.devices import _shutdown
    ds = DistanceSensor(17, 19)
    f = Device.pin_factory
    _shutdown()
    assert ds.closed
    assert not f.pins
    assert Device.pin_factory is None
    # Shutdown must be idempotent
    _shutdown()



from gpiozero import DistanceSensor, LED
from signal import pause

sensor = DistanceSensor(23, 24, max_distance=1, threshold_distance=0.2)
led = LED(16)

sensor.when_in_range = led.on
sensor.when_out_of_range = led.off

pause()



# vim: set fileencoding=utf-8:
#
# GPIO Zero: a library for controlling the Raspberry Pi's GPIO pins
#
# Copyright (c) 2016-2021 Dave Jones <dave@waveform.org.uk>
#
# SPDX-License-Identifier: BSD-3-Clause

from __future__ import (
    unicode_literals,
    print_function,
    absolute_import,
    division,
    )
str = type('')


import sys
import pytest
import warnings
from threading import Event, Thread

from gpiozero import Device
from gpiozero.pins.mock import MockFactory, MockPWMPin


# NOTE: Work-around for python versions <3.4: in these versions the
# resetwarnings function in the warnings module doesn't do much (or doesn't do
# what most would expect). Warnings that have already been triggered still
# won't be re-triggered even after the call. To work-around this we set the
# default filter to "always" on these versions before running any tests. Note
# that this does mean you should run resetwarnings() whenever using
# catch_warnings()
if sys.version_info[:2] < (3, 4):
    warnings.simplefilter('always')

@pytest.yield_fixture()
def no_default_factory(request):
    save_pin_factory = Device.pin_factory
    Device.pin_factory = None
    try:
        yield None
    finally:
        Device.pin_factory = save_pin_factory

@pytest.yield_fixture(scope='function')
def mock_factory(request):
    save_factory = Device.pin_factory
    Device.pin_factory = MockFactory()
    try:
        yield Device.pin_factory
        # This reset() may seem redundant given we're re-constructing the
        # factory for each function that requires it but MockFactory (via
        # LocalFactory) stores some info at the class level which reset()
        # clears.
    finally:
        if Device.pin_factory is not None:
            Device.pin_factory.reset()
        Device.pin_factory = save_factory

@pytest.fixture()
def pwm(request, mock_factory):
    mock_factory.pin_class = MockPWMPin

class ThreadedTest(Thread):
    def __init__(self, test_fn, *args, **kwargs):
        self._fn = test_fn
        self._args = args
        self._kwargs = kwargs
        self._result = None
        super(ThreadedTest, self).__init__()
        self.daemon = True
        self.start()

    def run(self):
        self._result = self._fn(*self._args, **self._kwargs)

    @property
    def result(self):
        self.join(1)
        if not self.is_alive():
            return self._result
        else:
            raise RuntimeError('test thread did not finish')




# vim: set fileencoding=utf-8:
#
# GPIO Zero: a library for controlling the Raspberry Pi's GPIO pins
#
# Copyright (c) 2015-2021 Dave Jones <dave@waveform.org.uk>
# Copyright (c) 2016-2020 Andrew Scheller <github@loowis.durge.org>
#
# SPDX-License-Identifier: BSD-3-Clause

from __future__ import (
    unicode_literals,
    absolute_import,
    print_function,
    division,
    )
try:
    range = xrange
except NameError:
    pass
nstr = str
str = type('')

import io
import os
import sys
import mmap
import errno
import struct
import select
import warnings
from time import sleep
from threading import Thread, Event, RLock
from collections import Counter
try:
    from queue import Queue, Empty
except ImportError:
    from Queue import Queue, Empty

from .local import LocalPiPin, LocalPiFactory
from ..exc import (
    PinInvalidPull,
    PinInvalidEdges,
    PinInvalidFunction,
    PinFixedPull,
    PinSetInput,
    )


def dt_resolve_alias(alias, root='/proc/device-tree'):
    """
    Returns the full path of a device-tree alias. For example:

        >>> dt_resolve_alias('gpio')
        '/proc/device-tree/soc/gpio@7e200000'
        >>> dt_resolve_alias('ethernet0', root='/proc/device-tree')
        '/proc/device-tree/scb/ethernet@7d580000'
    """
    # XXX Change this return a pathlib.Path when we drop 2.x
    filename = os.path.join(root, 'aliases', alias)
    with io.open(filename, 'rb') as f:
        node, tail = f.read().split(b'\0', 1)
        fs_encoding = sys.getfilesystemencoding()
        return os.path.join(root, node.decode(fs_encoding).lstrip('/'))

def dt_peripheral_reg(node, root='/proc/device-tree'):
    """
    Returns the :class:`range` covering the registers of the specified *node*
    of the device-tree, mapped to the CPU's address space. For example:

        >>> reg = dt_peripheral_reg(dt_resolve_alias('gpio'))
        >>> '%#x..%#x' % (reg.start, reg.stop)
        '0xfe200000..0xfe2000b4'
        >>> hex(dt_peripheral_reg(dt_resolve_alias('ethernet0')).start)
        '0xfd580000'
    """
    # Returns a tuple of (address-cells, size-cells) for *node*
    def _cells(node):
        with io.open(os.path.join(node, '#address-cells'), 'rb') as f:
            address_cells = struct.unpack(nstr('>L'), f.read())[0]
        with io.open(os.path.join(node, '#size-cells'), 'rb') as f:
            size_cells = struct.unpack(nstr('>L'), f.read())[0]
        return (address_cells, size_cells)

    # Returns a generator function which, given a file-like object *source*
    # iteratively decodes it, yielding a tuple of values from it. Each tuple
    # contains one integer for each specified *length*, which is the number of
    # 32-bit device-tree cells that make up that value.
    def _reader(*lengths):
        structs = [struct.Struct(nstr('>{cells}L'.format(cells=cells)))
                   for cells in lengths]
        offsets = [sum(s.size for s in structs[:i])
                   for i in range(len(structs))]
        buf_len = sum(s.size for s in structs)

        def fn(source):
            while True:
                buf = source.read(buf_len)
                if not buf:
                    break
                elif len(buf) < buf_len:
                    raise IOError('failed to read {buf_len} bytes'.format(
                        buf_len=buf_len))
                row = ()
                for offset, s in zip(offsets, structs):
                    cells = s.unpack_from(buf, offset)
                    value = 0
                    for cell in cells:
                        value = (value << 32) | cell
                    row += (value,)
                yield row
        return fn

    # Returns a list of (child-range, parent-range) tuples for *node*
    def _ranges(node):
        child_cells, size_cells = _cells(node)
        parent = os.path.dirname(node)
        parent_cells, _ = _cells(parent)
        ranges_reader = _reader(child_cells, parent_cells, size_cells)
        with io.open(os.path.join(node, 'ranges'), 'rb') as f:
            return [
                (range(child_base, child_base + size),
                 range(parent_base, parent_base + size))
                for child_base, parent_base, size in ranges_reader(f)
            ]

    # XXX Replace all this gubbins with pathlib.Path stuff once we drop 2.x
    node = os.path.join(root, node)
    parent = os.path.dirname(node)
    child_cells, size_cells = _cells(parent)
    reg_reader = _reader(child_cells, size_cells)
    with io.open(os.path.join(node, 'reg'), 'rb') as f:
        base, size = list(reg_reader(f))[0]
    while parent != root:
        # Iterate up the hierarchy, resolving the base address as we go
        if os.path.exists(os.path.join(parent, 'ranges')):
            for child_range, parent_range in _ranges(parent):
                if base in child_range:
                    # XXX Can't use .start here as python2's crappy xrange
                    # lacks it; change this when we drop 2.x!
                    base += parent_range[0] - child_range[0]
                    break
        parent = os.path.dirname(parent)
    return range(base, base + size)


class GPIOMemory(object):
    GPIO_BASE_OFFSET = 0x200000
    PERI_BASE_OFFSET = {
        'BCM2835': 0x20000000,
        'BCM2836': 0x3f000000,
        'BCM2837': 0x3f000000,
        'BCM2711': 0xfe000000,
    }

    # From BCM2835 data-sheet, p.91
    GPFSEL_OFFSET   = 0x00 >> 2
    GPSET_OFFSET    = 0x1c >> 2
    GPCLR_OFFSET    = 0x28 >> 2
    GPLEV_OFFSET    = 0x34 >> 2
    GPEDS_OFFSET    = 0x40 >> 2
    GPREN_OFFSET    = 0x4c >> 2
    GPFEN_OFFSET    = 0x58 >> 2
    GPHEN_OFFSET    = 0x64 >> 2
    GPLEN_OFFSET    = 0x70 >> 2
    GPAREN_OFFSET   = 0x7c >> 2
    GPAFEN_OFFSET   = 0x88 >> 2
    GPPUD_OFFSET    = 0x94 >> 2
    GPPUDCLK_OFFSET = 0x98 >> 2
    # pull-control registers for BCM2711
    GPPUPPDN_OFFSET = 0xe4 >> 2

    def __init__(self, soc):
        try:
            self.fd = os.open('/dev/gpiomem', os.O_RDWR | os.O_SYNC)
        except OSError:
            try:
                self.fd = os.open('/dev/mem', os.O_RDWR | os.O_SYNC)
            except OSError:
                raise IOError(
                    'unable to open /dev/gpiomem or /dev/mem; '
                    'upgrade your kernel or run as root')
            else:
                offset = self.gpio_base(soc)
        else:
            offset = 0
        self.mem = mmap.mmap(self.fd, 4096, offset=offset)
        # Register reads and writes must be in native format (otherwise
        # struct resorts to individual byte reads/writes and you can't hit
        # half a register :). For arm64 compat we have to figure out what the
        # native unsigned 32-bit type is...
        try:
            self.reg_fmt = {
                struct.calcsize(fmt): fmt
                for fmt in (nstr('@I'), nstr('@L'))
            }[4]
        except KeyError:
            raise RuntimeError('unable to find native unsigned 32-bit type')

    def close(self):
        self.mem.close()
        os.close(self.fd)

    def gpio_base(self, soc):
        try:
            # XXX Replace this with .start when 2.x is dropped
            return dt_peripheral_reg(dt_resolve_alias('gpio'))[0]
        except IOError:
            try:
                return self.PERI_BASE_OFFSET[soc] + self.GPIO_BASE_OFFSET
            except KeyError:
                pass
        raise IOError('unable to determine gpio base')

    def __getitem__(self, index):
        return struct.unpack_from(self.reg_fmt, self.mem, index * 4)[0]

    def __setitem__(self, index, value):
        struct.pack_into(self.reg_fmt, self.mem, index * 4, value)


class GPIOFS(object):
    GPIO_PATH = '/sys/class/gpio'

    def __init__(self, factory, queue):
        self._lock = RLock()
        self._exports = {}
        self._thread = NativeWatchThread(factory, queue)

    def close(self):
        # We *could* track the stuff we've exported and unexport it here, but
        # exports are a system global resource. We can't guarantee that some
        # other process isn't relying on something we've exported. In other
        # words, once exported it's *never* safe to unexport something. The
        # unexport method below is largely provided for debugging and testing.
        if self._thread is not None:
            self._thread.close()
            self._thread = None

    def path(self, name):
        return os.path.join(self.GPIO_PATH, name)

    def path_value(self, pin):
        return self.path('gpio%d/value' % pin)

    def path_dir(self, pin):
        return self.path('gpio%d/direction' % pin)

    def path_edge(self, pin):
        return self.path('gpio%d/edge' % pin)

    def exported(self, pin):
        return pin in self._exports

    def export(self, pin):
        with self._lock:
            try:
                result = self._exports[pin]
            except KeyError:
                result = None
                # Dirty hack to wait for udev to set permissions on
                # gpioN/value; there's no other way around this as there's no
                # synchronous mechanism for setting permissions on sysfs
                for i in range(10):
                    try:
                        # Must be O_NONBLOCK for use with epoll in edge
                        # triggered mode
                        result = os.open(self.path_value(pin),
                                         os.O_RDONLY | os.O_NONBLOCK)
                    except IOError as e:
                        if e.errno == errno.ENOENT:
                            with io.open(self.path('export'), 'wb') as f:
                                f.write(str(pin).encode('ascii'))
                        elif e.errno == errno.EACCES:
                            sleep(i / 100)
                        else:
                            raise
                    else:
                        self._exports[pin] = result
                        break
                # Same for gpioN/edge. It must exist by this point but the
                # chmod -R may not have reached it yet...
                for i in range(10):
                    try:
                        with io.open(self.path_edge(pin), 'w+b'):
                            pass
                    except IOError as e:
                        if e.errno == errno.EACCES:
                            sleep(i / 100)
                        else:
                            raise
                if result is None:
                    raise RuntimeError('failed to export pin %d' % pin)
            return result

    def unexport(self, pin):
        with self._lock:
            try:
                os.close(self._exports.pop(pin))
            except KeyError:
                # unexport should be idempotent
                pass
            else:
                try:
                    with io.open(self.path('unexport'), 'wb') as f:
                        f.write(str(pin).encode('ascii'))
                except IOError as e:
                    if e.errno == errno.EINVAL:
                        # Someone already unexported it; ignore the error
                        pass

    def watch(self, pin):
        with self._lock:
            self._thread.watch(self.export(pin), pin)

    def unwatch(self, pin):
        with self._lock:
            try:
                self._thread.unwatch(self._exports[pin])
            except KeyError:
                pass


class NativeWatchThread(Thread):
    def __init__(self, factory, queue):
        super(NativeWatchThread, self).__init__(
            target=self._run, args=(factory, queue))
        self.daemon = True
        self._stop_evt = Event()
        # XXX Make this compatible with BSDs with poll() option?
        self._epoll = select.epoll()
        self._watches = {}
        self.start()

    def close(self):
        self._stop_evt.set()
        self.join()
        self._epoll.close()

    def watch(self, fd, pin):
        self._watches[fd] = pin
        flags = select.EPOLLIN | select.EPOLLPRI | select.EPOLLET
        self._epoll.register(fd, flags)

    def unwatch(self, fd):
        self._epoll.unregister(fd)
        fd = self._watches.pop(fd, None)

    def _run(self, factory, queue):
        ticks = factory.ticks
        while not self._stop_evt.wait(0):
            for fd, event in self._epoll.poll(0.01):
                when = ticks()
                state = os.read(fd, 1) == b'1'
                os.lseek(fd, 0, 0)
                try:
                    queue.put((self._watches[fd], when, state))
                except KeyError:
                    pass


class NativeDispatchThread(Thread):
    def __init__(self, factory, queue):
        super(NativeDispatchThread, self).__init__(
            target=self._run, args=(factory, queue))
        self.daemon = True
        self._stop_evt = Event()
        self.start()

    def close(self):
        self._stop_evt.set()
        self.join()

    def _run(self, factory, queue):
        pins = factory.pins
        while not self._stop_evt.wait(0):
            try:
                num, ticks, state = queue.get(timeout=0.1)
            except Empty:
                continue
            try:
                pin = pins[num]
            except KeyError:
                pass
            else:
                if (
                        pin._bounce is None or pin._last_call is None or
                        factory.ticks_diff(ticks, pin._last_call) > pin._bounce
                ):
                    pin._call_when_changed(ticks, state)
                    pin._last_call = ticks


class NativeFactory(LocalPiFactory):
    """
    Extends :class:`~gpiozero.pins.local.LocalPiFactory`. Uses a built-in pure
    Python implementation to interface to the Pi's GPIO pins. This is the
    default pin implementation if no third-party libraries are discovered.

    .. warning::

        This implementation does *not* currently support PWM. Attempting to
        use any class which requests PWM will raise an exception.

    You can construct native pin instances manually like so::

        from gpiozero.pins.native import NativeFactory
        from gpiozero import LED

        factory = NativeFactory()
        led = LED(12, pin_factory=factory)
    """
    def __init__(self):
        super(NativeFactory, self).__init__()
        queue = Queue()
        self.mem = GPIOMemory(self.pi_info.soc)
        self.fs = GPIOFS(self, queue)
        self.dispatch = NativeDispatchThread(self, queue)
        if self.pi_info.soc == 'BCM2711':
            self.pin_class = Native2711Pin
        else:
            self.pin_class = Native2835Pin

    def close(self):
        if self.dispatch is not None:
            self.dispatch.close()
            self.dispatch = None
        super(NativeFactory, self).close()
        if self.fs is not None:
            self.fs.close()
            self.fs = None
        if self.mem is not None:
            self.mem.close()
            self.mem = None


class NativePin(LocalPiPin):
    """
    Extends :class:`~gpiozero.pins.local.LocalPiPin`. Native pin
    implementation. See :class:`NativeFactory` for more information.
    """
    GPIO_FUNCTIONS = {
        'input':   0b000,
        'output':  0b001,
        'alt0':    0b100,
        'alt1':    0b101,
        'alt2':    0b110,
        'alt3':    0b111,
        'alt4':    0b011,
        'alt5':    0b010,
        }

    GPIO_FUNCTION_NAMES = {v: k for (k, v) in GPIO_FUNCTIONS.items()}

    def __init__(self, factory, number):
        super(NativePin, self).__init__(factory, number)
        self._reg_init(factory, number)
        self._last_call = None
        self._when_changed = None
        self._change_thread = None
        self._change_event = Event()
        self.function = 'input'
        self.pull = 'up' if self.factory.pi_info.pulled_up(repr(self)) else 'floating'
        self.bounce = None
        self.edges = 'none'

    def _reg_init(self, factory, number):
        self._func_offset = self.factory.mem.GPFSEL_OFFSET + (number // 10)
        self._func_shift = (number % 10) * 3
        self._set_offset = self.factory.mem.GPSET_OFFSET + (number // 32)
        self._set_shift = number % 32
        self._clear_offset = self.factory.mem.GPCLR_OFFSET + (number // 32)
        self._clear_shift = number % 32
        self._level_offset = self.factory.mem.GPLEV_OFFSET + (number // 32)
        self._level_shift = number % 32
        self._edge_offset = self.factory.mem.GPEDS_OFFSET + (number // 32)
        self._edge_shift = number % 32
        self._rising_offset = self.factory.mem.GPREN_OFFSET + (number // 32)
        self._rising_shift = number % 32
        self._falling_offset = self.factory.mem.GPFEN_OFFSET + (number // 32)
        self._falling_shift = number % 32

    def close(self):
        self.edges = 'none'
        self.frequency = None
        self.when_changed = None
        self.function = 'input'
        self.pull = 'up' if self.factory.pi_info.pulled_up(repr(self)) else 'floating'

    def _get_function(self):
        return self.GPIO_FUNCTION_NAMES[(self.factory.mem[self._func_offset] >> self._func_shift) & 7]

    def _set_function(self, value):
        try:
            value = self.GPIO_FUNCTIONS[value]
        except KeyError:
            raise PinInvalidFunction('invalid function "%s" for pin %r' % (value, self))
        self.factory.mem[self._func_offset] = (
            self.factory.mem[self._func_offset]
            & ~(7 << self._func_shift)
            | (value << self._func_shift)
            )

    def _get_state(self):
        return bool(self.factory.mem[self._level_offset] & (1 << self._level_shift))

    def _set_state(self, value):
        if self.function == 'input':
            raise PinSetInput('cannot set state of pin %r' % self)
        if value:
            self.factory.mem[self._set_offset] = 1 << self._set_shift
        else:
            self.factory.mem[self._clear_offset] = 1 << self._clear_shift

    def _get_pull(self):
        raise NotImplementedError

    def _set_pull(self, value):
        raise NotImplementedError

    def _get_bounce(self):
        return self._bounce

    def _set_bounce(self, value):
        self._bounce = None if value is None else float(value)

    def _get_edges(self):
        try:
            with io.open(self.factory.fs.path_edge(self.number), 'r') as f:
                return f.read().strip()
        except IOError as e:
            if e.errno == errno.ENOENT:
                return 'none'
            else:
                raise

    def _set_edges(self, value):
        if value != 'none':
            self.factory.fs.export(self.number)
        try:
            with io.open(self.factory.fs.path_edge(self.number), 'w') as f:
                f.write(value)
        except IOError as e:
            if e.errno == errno.ENOENT and value == 'none':
                pass
            elif e.errno == errno.EINVAL:
                raise PinInvalidEdges('invalid edge specification "%s" for pin %r' % self)
            else:
                raise

    def _enable_event_detect(self):
        self.factory.fs.watch(self.number)
        self._last_call = None

    def _disable_event_detect(self):
        self.factory.fs.unwatch(self.number)


class Native2835Pin(NativePin):
    """
    Extends :class:`NativePin` for Pi hardware prior to the Pi 4 (Pi 0, 1, 2,
    3, and 3+).
    """
    GPIO_PULL_UPS = {
        'up':       0b10,
        'down':     0b01,
        'floating': 0b00,
    }

    GPIO_PULL_UP_NAMES = {v: k for (k, v) in GPIO_PULL_UPS.items()}

    def _reg_init(self, factory, number):
        super(Native2835Pin, self)._reg_init(factory, number)
        self._pull_offset = self.factory.mem.GPPUDCLK_OFFSET + (number // 32)
        self._pull_shift = number % 32
        self._pull = 'floating'

    def _get_pull(self):
        return self.GPIO_PULL_UP_NAMES[self._pull]

    def _set_pull(self, value):
        if self.function != 'input':
            raise PinFixedPull('cannot set pull on non-input pin %r' % self)
        if value != 'up' and self.factory.pi_info.pulled_up(repr(self)):
            raise PinFixedPull('%r has a physical pull-up resistor' % self)
        try:
            value = self.GPIO_PULL_UPS[value]
        except KeyError:
            raise PinInvalidPull('invalid pull direction "%s" for pin %r' % (value, self))
        self.factory.mem[self.factory.mem.GPPUD_OFFSET] = value
        sleep(0.000000214)
        self.factory.mem[self._pull_offset] = 1 << self._pull_shift
        sleep(0.000000214)
        self.factory.mem[self.factory.mem.GPPUD_OFFSET] = 0
        self.factory.mem[self._pull_offset] = 0
        self._pull = value


class Native2711Pin(NativePin):
    """
    Extends :class:`NativePin` for Pi 4 hardware (Pi 4, CM4, Pi 400 at the time
    of writing).
    """
    GPIO_PULL_UPS = {
        'up':       0b01,
        'down':     0b10,
        'floating': 0b00,
    }

    GPIO_PULL_UP_NAMES = {v: k for (k, v) in GPIO_PULL_UPS.items()}

    def _reg_init(self, factory, number):
        super(Native2711Pin, self)._reg_init(factory, number)
        self._pull_offset = self.factory.mem.GPPUPPDN_OFFSET + (number // 16)
        self._pull_shift = (number % 16) * 2

    def _get_pull(self):
        pull = (self.factory.mem[self._pull_offset] >> self._pull_shift) & 3
        return self.GPIO_PULL_UP_NAMES[pull]

    def _set_pull(self, value):
        if self.function != 'input':
            raise PinFixedPull('cannot set pull on non-input pin %r' % self)
        if value != 'up' and self.factory.pi_info.pulled_up(repr(self)):
            raise PinFixedPull('%r has a physical pull-up resistor' % self)
        try:
            value = self.GPIO_PULL_UPS[value]
        except KeyError:
            raise PinInvalidPull('invalid pull direction "%s" for pin %r' % (value, self))
        self.factory.mem[self._pull_offset] = (
            self.factory.mem[self._pull_offset]
            & ~(3 << self._pull_shift)
            | (value << self._pull_shift)
            )



from gpiozero import AngularServo
from time import sleep

servo = AngularServo(17, min_angle=-90, max_angle=90)

while True:
    servo.angle = -90
    sleep(2)
    servo.angle = -45
    sleep(2)
    servo.angle = 0
    sleep(2)
    servo.angle = 45
    sleep(2)
    servo.angle = 90
    sleep(2)



from gpiozero import RGBLED, MCP3008
from gpiozero.tools import zip_values
from signal import pause

led = RGBLED(2, 3, 4)
red_pot = MCP3008(0)
green_pot = MCP3008(1)
blue_pot = MCP3008(2)

led.source = zip_values(red_pot, green_pot, blue_pot)

pause()



import gpiozero
from gpiozero import TrafficHat
from gpiozero.pins.pigpio import PiGPIOFactory
from time import sleep

remote_factory = PiGPIOFactory(host='192.168.1.3')

th_1 = TrafficHat()  # traffic hat using local pins
th_2 = TrafficHat(pin_factory=remote_factory)  # traffic hat on 192.168.1.3 using remote pins



from gpiozero import TrafficLights
from time import sleep
from signal import pause

lights = TrafficLights(2, 3, 4)

def traffic_light_sequence():
    while True:
        yield (0, 0, 1) # green
        sleep(10)
        yield (0, 1, 0) # amber
        sleep(1)
        yield (1, 0, 0) # red
        sleep(10)
        yield (1, 1, 0) # red+amber
        sleep(1)

lights.source = traffic_light_sequence()

pause()



from gpiozero import LEDBoard
from time import sleep
from signal import pause

leds = LEDBoard(5, 6, 13, 19, 26)

leds.on()
sleep(1)
leds.off()
sleep(1)
leds.value = (1, 0, 1, 0, 1)
sleep(1)
leds.blink()

pause()



from gpiozero import Robot, MCP3008
from gpiozero.tools import zip_values
from signal import pause

robot = Robot(left=(4, 14), right=(17, 18))

left_pot = MCP3008(0)
right_pot = MCP3008(1)

robot.source = zip_values(left_pot, right_pot)

pause()



# vim: set fileencoding=utf-8:
#
# GPIO Zero: a library for controlling the Raspberry Pi's GPIO pins
#
# Copyright (c) 2016-2021 Dave Jones <dave@waveform.org.uk>
# Copyright (c) 2018-2021 Ben Nuttall <ben@bennuttall.com>
# Copyright (c) 2016 Andrew Scheller <github@loowis.durge.org>
#
# SPDX-License-Identifier: BSD-3-Clause

from __future__ import (
    unicode_literals,
    print_function,
    absolute_import,
    division,
    )
nstr = str
str = type('')

import inspect
import weakref
from functools import wraps, partial
from threading import Event
from collections import deque
try:
    from statistics import median
except ImportError:
    from .compat import median
import warnings

from .threads import GPIOThread
from .exc import (
    BadEventHandler,
    BadWaitTime,
    BadQueueLen,
    DeviceClosed,
    CallbackSetToNone,
    )

callback_warning = (
    'The callback was set to None. This may have been unintentional '
    'e.g. btn.when_pressed = pressed() instead of btn.when_pressed = pressed'
)

class ValuesMixin(object):
    """
    Adds a :attr:`values` property to the class which returns an infinite
    generator of readings from the :attr:`~Device.value` property. There is
    rarely a need to use this mixin directly as all base classes in GPIO Zero
    include it.

    .. note::

        Use this mixin *first* in the parent class list.
    """

    @property
    def values(self):
        """
        An infinite iterator of values read from :attr:`value`.
        """
        while True:
            try:
                yield self.value
            except DeviceClosed:
                break


class SourceMixin(object):
    """
    Adds a :attr:`source` property to the class which, given an iterable or a
    :class:`ValuesMixin` descendent, sets :attr:`~Device.value` to each member
    of that iterable until it is exhausted. This mixin is generally included in
    novel output devices to allow their state to be driven from another device.

    .. note::

        Use this mixin *first* in the parent class list.
    """

    def __init__(self, *args, **kwargs):
        self._source = None
        self._source_thread = None
        self._source_delay = 0.01
        super(SourceMixin, self).__init__(*args, **kwargs)

    def close(self):
        self.source = None
        super(SourceMixin, self).close()

    def _copy_values(self, source):
        for v in source:
            self.value = v
            if self._source_thread.stopping.wait(self._source_delay):
                break

    @property
    def source_delay(self):
        """
        The delay (measured in seconds) in the loop used to read values from
        :attr:`source`. Defaults to 0.01 seconds which is generally sufficient
        to keep CPU usage to a minimum while providing adequate responsiveness.
        """
        return self._source_delay

    @source_delay.setter
    def source_delay(self, value):
        if value < 0:
            raise BadWaitTime('source_delay must be 0 or greater')
        self._source_delay = float(value)

    @property
    def source(self):
        """
        The iterable to use as a source of values for :attr:`value`.
        """
        return self._source

    @source.setter
    def source(self, value):
        if getattr(self, '_source_thread', None):
            self._source_thread.stop()
        self._source_thread = None
        if isinstance(value, ValuesMixin):
            value = value.values
        self._source = value
        if value is not None:
            self._source_thread = GPIOThread(self._copy_values, (value,))
            self._source_thread.start()


class SharedMixin(object):
    """
    This mixin marks a class as "shared". In this case, the meta-class
    (GPIOMeta) will use :meth:`_shared_key` to convert the constructor
    arguments to an immutable key, and will check whether any existing
    instances match that key. If they do, they will be returned by the
    constructor instead of a new instance. An internal reference counter is
    used to determine how many times an instance has been "constructed" in this
    way.

    When :meth:`~Device.close` is called, an internal reference counter will be
    decremented and the instance will only close when it reaches zero.
    """
    _instances = {}

    def __del__(self):
        self._refs = 0
        super(SharedMixin, self).__del__()

    @classmethod
    def _shared_key(cls, *args, **kwargs):
        """
        This is called with the constructor arguments to generate a unique
        key (which must be storable in a :class:`dict` and, thus, immutable
        and hashable) representing the instance that can be shared. This must
        be overridden by descendents.

        The default simply assumes all positional arguments are immutable and
        returns this as the key but this is almost never the "right" thing to
        do and almost all descendents should override this method.
        """
        # XXX Future 2.x version should change this to raise NotImplementedError
        return args


class event(object):
    """
    A descriptor representing a callable event on a class descending from
    :class:`EventsMixin`.

    Instances of this class are very similar to a :class:`property` but also
    deal with notifying the owning class when events are assigned (or
    unassigned) and wrapping callbacks implicitly as appropriate.
    """
    def __init__(self, doc=None):
        self.handlers = {}
        self.__doc__ = doc

    def __get__(self, instance, owner=None):
        if instance is None:
            return self
        else:
            return self.handlers.get(id(instance))

    def __set__(self, instance, value):
        if value is None:
            try:
                del self.handlers[id(instance)]
            except KeyError:
                warnings.warn(CallbackSetToNone(callback_warning))
        else:
            self.handlers[id(instance)] = instance._wrap_callback(value)
        enabled = any(
            obj.handlers.get(id(instance))
            for name in dir(type(instance))
            for obj in (getattr(type(instance), name),)
            if isinstance(obj, event)
        )
        instance._start_stop_events(enabled)


class EventsMixin(object):
    """
    Adds edge-detected :meth:`when_activated` and :meth:`when_deactivated`
    events to a device based on changes to the :attr:`~Device.is_active`
    property common to all devices. Also adds :meth:`wait_for_active` and
    :meth:`wait_for_inactive` methods for level-waiting.

    .. note::

        Note that this mixin provides no means of actually firing its events;
        call :meth:`_fire_events` in sub-classes when device state changes to
        trigger the events. This should also be called once at the end of
        initialization to set initial states.
    """
    def __init__(self, *args, **kwargs):
        super(EventsMixin, self).__init__(*args, **kwargs)
        self._active_event = Event()
        self._inactive_event = Event()
        self._last_active = None
        self._last_changed = self.pin_factory.ticks()

    def _all_events(self):
        """
        Generator function which yields all :class:`event` instances defined
        against this class.
        """
        for name in dir(type(self)):
            obj = getattr(type(self), name)
            if isinstance(obj, event):
                yield obj

    def close(self):
        for ev in self._all_events():
            try:
                del ev.handlers[id(self)]
            except KeyError:
                pass
        super(EventsMixin, self).close()

    def wait_for_active(self, timeout=None):
        """
        Pause the script until the device is activated, or the timeout is
        reached.

        :type timeout: float or None
        :param timeout:
            Number of seconds to wait before proceeding. If this is
            :data:`None` (the default), then wait indefinitely until the device
            is active.
        """
        return self._active_event.wait(timeout)

    def wait_for_inactive(self, timeout=None):
        """
        Pause the script until the device is deactivated, or the timeout is
        reached.

        :type timeout: float or None
        :param timeout:
            Number of seconds to wait before proceeding. If this is
            :data:`None` (the default), then wait indefinitely until the device
            is inactive.
        """
        return self._inactive_event.wait(timeout)

    when_activated = event(
        """
        The function to run when the device changes state from inactive to
        active.

        This can be set to a function which accepts no (mandatory) parameters,
        or a Python function which accepts a single mandatory parameter (with
        as many optional parameters as you like). If the function accepts a
        single mandatory parameter, the device that activated it will be passed
        as that parameter.

        Set this property to :data:`None` (the default) to disable the event.
        """)

    when_deactivated = event(
        """
        The function to run when the device changes state from active to
        inactive.

        This can be set to a function which accepts no (mandatory) parameters,
        or a Python function which accepts a single mandatory parameter (with
        as many optional parameters as you like). If the function accepts a
        single mandatory parameter, the device that deactivated it will be
        passed as that parameter.

        Set this property to :data:`None` (the default) to disable the event.
        """)

    @property
    def active_time(self):
        """
        The length of time (in seconds) that the device has been active for.
        When the device is inactive, this is :data:`None`.
        """
        if self._active_event.is_set():
            return self.pin_factory.ticks_diff(self.pin_factory.ticks(),
                                               self._last_changed)
        else:
            return None

    @property
    def inactive_time(self):
        """
        The length of time (in seconds) that the device has been inactive for.
        When the device is active, this is :data:`None`.
        """
        if self._inactive_event.is_set():
            return self.pin_factory.ticks_diff(self.pin_factory.ticks(),
                                               self._last_changed)
        else:
            return None

    def _wrap_callback(self, fn):
        # XXX In 2.x, move this to the event class above
        if not callable(fn):
            raise BadEventHandler('value must be None or a callable')
        # If fn is wrapped with partial (i.e. partial, partialmethod, or wraps
        # has been used to produce it) we need to dig out the "real" function
        # that's been wrapped along with all the mandatory positional args
        # used in the wrapper so we can test the binding
        args = ()
        wrapped_fn = fn
        while isinstance(wrapped_fn, partial):
            args = wrapped_fn.args + args
            wrapped_fn = wrapped_fn.func
        if inspect.isbuiltin(wrapped_fn):
            # We can't introspect the prototype of builtins. In this case we
            # assume that the builtin has no (mandatory) parameters; this is
            # the most reasonable assumption on the basis that pre-existing
            # builtins have no knowledge of gpiozero, and the sole parameter
            # we would pass is a gpiozero object
            return fn
        else:
            # Try binding ourselves to the argspec of the provided callable.
            # If this works, assume the function is capable of accepting no
            # parameters
            try:
                inspect.getcallargs(wrapped_fn, *args)
                return fn
            except TypeError:
                try:
                    # If the above fails, try binding with a single parameter
                    # (ourselves). If this works, wrap the specified callback
                    inspect.getcallargs(wrapped_fn, *(args + (self,)))
                    @wraps(fn)
                    def wrapper():
                        return fn(self)
                    return wrapper
                except TypeError:
                    raise BadEventHandler(
                        'value must be a callable which accepts up to one '
                        'mandatory parameter')

    def _fire_activated(self):
        # These methods are largely here to be overridden by descendents
        if self.when_activated:
            self.when_activated()

    def _fire_deactivated(self):
        # These methods are largely here to be overridden by descendents
        if self.when_deactivated:
            self.when_deactivated()

    def _fire_events(self, ticks, new_active):
        """
        This method should be called by descendents whenever the
        :attr:`~Device.is_active` property is likely to have changed (for
        example, in response to a pin's :attr:`~gpiozero.Pin.state` changing).

        The *ticks* parameter must be set to the time when the change occurred;
        this can usually be obtained from the pin factory's
        :meth:`gpiozero.Factory.ticks` method but some pin implementations will
        implicitly provide the ticks when an event occurs as part of their
        reporting mechanism.

        The *new_active* parameter must be set to the device's
        :attr:`~Device.is_active` value at the time indicated by *ticks* (which
        is not necessarily the value of :attr:`~Device.is_active` right now, if
        the pin factory provides means of reporting a pin's historical state).
        """
        old_active, self._last_active = self._last_active, new_active
        if old_active is None:
            # Initial "indeterminate" state; set events but don't fire
            # callbacks as there's not necessarily an edge
            if new_active:
                self._active_event.set()
            else:
                self._inactive_event.set()
        elif old_active != new_active:
            self._last_changed = ticks
            if new_active:
                self._inactive_event.clear()
                self._active_event.set()
                self._fire_activated()
            else:
                self._active_event.clear()
                self._inactive_event.set()
                self._fire_deactivated()

    def _start_stop_events(self, enabled):
        """
        This is a stub method that only exists to be overridden by descendents.
        It is called when :class:`event` properties are assigned (including
        when set to :data:`None) to permit the owning instance to activate or
        deactivate monitoring facilities.

        For example, if a descendent requires a background thread to monitor a
        device, it would be preferable to only run the thread if event handlers
        are present to respond to it.

        The *enabled* parameter is :data:`False` when all :class:`event`
        properties on the owning class are :data:`None`, and :data:`True`
        otherwise.
        """
        pass


class HoldMixin(EventsMixin):
    """
    Extends :class:`EventsMixin` to add the :attr:`when_held` event and the
    machinery to fire that event repeatedly (when :attr:`hold_repeat` is
    :data:`True`) at internals defined by :attr:`hold_time`.
    """
    def __init__(self, *args, **kwargs):
        self._hold_thread = None
        super(HoldMixin, self).__init__(*args, **kwargs)
        self._when_held = None
        self._held_from = None
        self._hold_time = 1
        self._hold_repeat = False
        self._hold_thread = HoldThread(self)

    def close(self):
        if self._hold_thread is not None:
            self._hold_thread.stop()
        self._hold_thread = None
        super(HoldMixin, self).close()

    def _fire_activated(self):
        super(HoldMixin, self)._fire_activated()
        self._hold_thread.holding.set()

    def _fire_deactivated(self):
        self._held_from = None
        super(HoldMixin, self)._fire_deactivated()

    def _fire_held(self):
        if self.when_held:
            self.when_held()

    when_held = event(
        """
        The function to run when the device has remained active for
        :attr:`hold_time` seconds.

        This can be set to a function which accepts no (mandatory) parameters,
        or a Python function which accepts a single mandatory parameter (with
        as many optional parameters as you like). If the function accepts a
        single mandatory parameter, the device that activated will be passed
        as that parameter.

        Set this property to :data:`None` (the default) to disable the event.
        """)

    @property
    def hold_time(self):
        """
        The length of time (in seconds) to wait after the device is activated,
        until executing the :attr:`when_held` handler. If :attr:`hold_repeat`
        is True, this is also the length of time between invocations of
        :attr:`when_held`.
        """
        return self._hold_time

    @hold_time.setter
    def hold_time(self, value):
        if value < 0:
            raise BadWaitTime('hold_time must be 0 or greater')
        self._hold_time = float(value)

    @property
    def hold_repeat(self):
        """
        If :data:`True`, :attr:`when_held` will be executed repeatedly with
        :attr:`hold_time` seconds between each invocation.
        """
        return self._hold_repeat

    @hold_repeat.setter
    def hold_repeat(self, value):
        self._hold_repeat = bool(value)

    @property
    def is_held(self):
        """
        When :data:`True`, the device has been active for at least
        :attr:`hold_time` seconds.
        """
        return self._held_from is not None

    @property
    def held_time(self):
        """
        The length of time (in seconds) that the device has been held for.
        This is counted from the first execution of the :attr:`when_held` event
        rather than when the device activated, in contrast to
        :attr:`~EventsMixin.active_time`. If the device is not currently held,
        this is :data:`None`.
        """
        if self._held_from is not None:
            return self.pin_factory.ticks_diff(self.pin_factory.ticks(),
                                               self._held_from)
        else:
            return None


class HoldThread(GPIOThread):
    """
    Extends :class:`GPIOThread`. Provides a background thread that repeatedly
    fires the :attr:`HoldMixin.when_held` event as long as the owning
    device is active.
    """
    def __init__(self, parent):
        super(HoldThread, self).__init__(
            target=self.held, args=(weakref.proxy(parent),))
        self.holding = Event()
        self.start()

    def held(self, parent):
        try:
            while not self.stopping.is_set():
                if self.holding.wait(0.1):
                    self.holding.clear()
                    while not (
                            self.stopping.is_set() or
                            parent._inactive_event.wait(parent.hold_time)
                            ):
                        if parent._held_from is None:
                            parent._held_from = parent.pin_factory.ticks()
                        parent._fire_held()
                        if not parent.hold_repeat:
                            break
        except ReferenceError:
            # Parent is dead; time to die!
            pass


class GPIOQueue(GPIOThread):
    """
    Extends :class:`GPIOThread`. Provides a background thread that monitors a
    device's values and provides a running *average* (defaults to median) of
    those values. If the *parent* device includes the :class:`EventsMixin` in
    its ancestry, the thread automatically calls
    :meth:`~EventsMixin._fire_events`.
    """
    def __init__(
            self, parent, queue_len=5, sample_wait=0.0, partial=False,
            average=median, ignore=None):
        assert callable(average)
        if queue_len < 1:
            raise BadQueueLen('queue_len must be at least one')
        if sample_wait < 0:
            raise BadWaitTime('sample_wait must be 0 or greater')
        if ignore is None:
            ignore = set()
        super(GPIOQueue, self).__init__(target=self.fill)
        self.queue = deque(maxlen=queue_len)
        self.partial = bool(partial)
        self.sample_wait = float(sample_wait)
        self.full = Event()
        self.parent = weakref.proxy(parent)
        self.average = average
        self.ignore = ignore

    @property
    def value(self):
        if not self.partial:
            self.full.wait()
        try:
            return self.average(self.queue)
        except (ZeroDivisionError, ValueError):
            # No data == inactive value
            return 0.0

    def fill(self):
        try:
            while not self.stopping.wait(self.sample_wait):
                value = self.parent._read()
                if value not in self.ignore:
                    self.queue.append(value)
                if not self.full.is_set() and len(self.queue) >= self.queue.maxlen:
                    self.full.set()
                if (self.partial or self.full.is_set()) and isinstance(self.parent, EventsMixin):
                    self.parent._fire_events(self.parent.pin_factory.ticks(), self.parent.is_active)
        except ReferenceError:
            # Parent is dead; time to die!
            pass



from gpiozero import Robot, MotionSensor
from signal import pause

robot = Robot(left=(4, 14), right=(17, 18))
pir = MotionSensor(5)

pir.when_motion = robot.forward
pir.when_no_motion = robot.stop

pause()



from gpiozero import Robot, MotionSensor
from gpiozero.tools import zip_values
from signal import pause

robot = Robot(left=(4, 14), right=(17, 18))
pir = MotionSensor(5)

robot.source = zip_values(pir, pir)

pause()



from gpiozero import LED, PingServer
from gpiozero.tools import negated
from signal import pause

green = LED(17)
red = LED(18)

google = PingServer('google.com')

google.when_activated = green.on
google.when_deactivated = green.off
red.source = negated(green)

pause()
